@@ -1,8 +1,8 @@
-/*	$calcurse: apoint.c,v 1.28 2008/12/15 20:02:00 culot Exp $	*/
+/*	$calcurse: apoint.c,v 1.33 2009/01/03 21:32:11 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2008 Frederic Culot
+ * Copyright (c) 2004-2009 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,19 +38,75 @@
 #include "recur.h"
 #include "keys.h"
 #include "calendar.h"
+#include "mem.h"
 #include "apoint.h"
 
-apoint_llist_t *alist_p;
-static int hilt = 0;
+apoint_llist_t             *alist_p;
+static apoint_llist_node_t  bkp_cut_apoint;
+static int                  hilt;
 
-int
+void
+apoint_free_bkp (erase_flag_e flag)
+{
+  if (bkp_cut_apoint.mesg)
+    {
+      mem_free (bkp_cut_apoint.mesg);
+      bkp_cut_apoint.mesg = 0;
+    }
+  if (bkp_cut_apoint.note)
+    {
+      if (flag == ERASE_FORCE)
+        erase_note (&bkp_cut_apoint.note, ERASE_FORCE);
+      else
+        {
+          mem_free (bkp_cut_apoint.note);
+          bkp_cut_apoint.note = 0;
+        }
+    }
+}
+
+static void
+apoint_dup (apoint_llist_node_t *in, apoint_llist_node_t *bkp)
+{
+  EXIT_IF (!in || !bkp, _("null pointer"));
+
+  bkp->start = in->start;
+  bkp->dur = in->dur;
+  bkp->state = in->state;
+  bkp->mesg = mem_strdup (in->mesg);
+  if (in->note)
+    bkp->note = mem_strdup (in->note);
+}
+
+void
 apoint_llist_init (void)
 {
-  alist_p = (apoint_llist_t *) malloc (sizeof (apoint_llist_t));
+  alist_p = (apoint_llist_t *) mem_malloc (sizeof (apoint_llist_t));
   alist_p->root = NULL;
   pthread_mutex_init (&(alist_p->mutex), NULL);
+}
 
-  return (0);
+/*
+ * Called before exit to free memory associated with the appointments linked
+ * list. No need to be thread safe, as only the main process remains when
+ * calling this function.
+ */
+void
+apoint_llist_free (void)
+{
+  apoint_llist_node_t *o, **i;
+
+  i = &alist_p->root;
+  while (*i)
+    {
+      o = *i;
+      *i = o->next;
+      mem_free (o->mesg);
+      if (o->note)
+        mem_free (o->note);
+      mem_free (o);
+    }
+  mem_free (alist_p);
 }
 
 /* Sets which appointment is highlighted. */
@@ -84,9 +140,9 @@ apoint_new (char *mesg, char *note, long start, long dur, char state)
 {
   apoint_llist_node_t *o, **i;
 
-  o = (apoint_llist_node_t *) malloc (sizeof (apoint_llist_node_t));
-  o->mesg = strdup (mesg);
-  o->note = (note != NULL) ? strdup (note) : NULL;
+  o = (apoint_llist_node_t *) mem_malloc (sizeof (apoint_llist_node_t));
+  o->mesg = mem_strdup (mesg);
+  o->note = (note != NULL) ? mem_strdup (note) : NULL;
   o->state = state;
   o->start = start;
   o->dur = dur;
@@ -131,8 +187,6 @@ apoint_add (void)
   char item_time[LTIME] = "";
   char item_mesg[BUFSIZ] = "";
   long apoint_duration = 0, apoint_start;
-  apoint_llist_node_t *apoint_pointeur;
-  struct event_s *event_pointeur;
   unsigned heures, minutes;
   unsigned end_h, end_m;
   int is_appointment = 1;
@@ -154,7 +208,7 @@ apoint_add (void)
 	      (void)wgetch (win[STA].p);
 	    }
 	  else
-	    sscanf (item_time, "%u:%u", &heures, &minutes);
+	    (void)sscanf (item_time, "%u:%u", &heures, &minutes);
 	}
       else
 	return;
@@ -183,7 +237,7 @@ apoint_add (void)
 		apoint_duration = atoi (item_time);
 	      else if (check_time (item_time) == 1)
 		{
-		  sscanf (item_time, "%u:%u", &end_h, &end_m);
+		  (void)sscanf (item_time, "%u:%u", &end_h, &end_m);
 		  if (end_h < heures)
 		    {
 		      apoint_duration = MININSEC - minutes + end_m
@@ -207,20 +261,19 @@ apoint_add (void)
       if (is_appointment)
 	{
 	  apoint_start = date2sec (*calendar_get_slctd_day (), heures, minutes);
-	  apoint_pointeur = apoint_new (item_mesg, 0L, apoint_start,
-					min2sec (apoint_duration), 0L);
+	  (void)apoint_new (item_mesg, 0L, apoint_start,
+                            min2sec (apoint_duration), 0L);
 	  if (notify_bar ())
 	    notify_check_added (item_mesg, apoint_start, 0L);
 	}
       else
-	event_pointeur = event_new (item_mesg, 0L,
-				    date2sec (*calendar_get_slctd_day (), 12,
-					      0), Id);
+	(void)event_new (item_mesg, 0L,
+                         date2sec (*calendar_get_slctd_day (), 12, 0), Id);
 
       if (hilt == 0)
 	hilt++;
     }
-  erase_status_bar ();
+  wins_erase_status_bar ();
 }
 
 /* Delete an item from the appointment list. */
@@ -246,7 +299,7 @@ apoint_delete (conf_t *conf, unsigned *nb_events, unsigned *nb_apoints)
 	go_for_deletion = true;
       else
 	{
-	  erase_status_bar ();
+	  wins_erase_status_bar ();
 	  return;
 	}
     }
@@ -276,14 +329,72 @@ apoint_delete (conf_t *conf, unsigned *nb_events, unsigned *nb_apoints)
 
 	  if (hilt > 1)
 	    hilt--;
-	  if (apad->first_onscreen >= to_be_removed)
-	    apad->first_onscreen = apad->first_onscreen - to_be_removed;
+	  if (apad.first_onscreen >= to_be_removed)
+	    apad.first_onscreen = apad.first_onscreen - to_be_removed;
 	  if (nb_items == 1)
 	    hilt = 0;
 	}
     }
 }
 
+/* Cut an item, so that it can be pasted somewhere else later. */
+int
+apoint_cut (unsigned *nb_events, unsigned *nb_apoints)
+{
+  const int NBITEMS = *nb_apoints + *nb_events;
+  int item_type, to_be_removed;  
+  long date;
+
+  if (NBITEMS == 0)
+    return 0;
+
+  to_be_removed = 0;
+  date = calendar_get_slctd_day_sec ();
+  item_type = day_cut_item (date, hilt);
+  if (item_type == EVNT || item_type == RECUR_EVNT)
+    {
+      (*nb_events)--;
+      to_be_removed = 1;
+    }
+  else if (item_type == APPT || item_type == RECUR_APPT)
+    {
+      (*nb_apoints)--;
+      to_be_removed = 3;
+    }
+  else
+    EXIT (_("no such type"));
+  /* NOTREACHED */
+  
+  if (hilt > 1)
+    hilt--;
+  if (apad.first_onscreen >= to_be_removed)
+    apad.first_onscreen = apad.first_onscreen - to_be_removed;
+  if (NBITEMS == 1)
+    hilt = 0;
+
+  return item_type;
+}
+
+/* Paste a previously cut item. */
+void
+apoint_paste (unsigned *nb_events, unsigned *nb_apoints, int cut_item_type)
+{
+  int item_type;
+  long date;
+
+  date = calendar_get_slctd_day_sec ();
+  item_type = day_paste_item (date, cut_item_type);
+  if (item_type == EVNT || item_type == RECUR_EVNT)
+    (*nb_events)++;
+  else if (item_type == APPT || item_type == RECUR_APPT)
+    (*nb_apoints)++;
+  else
+    return;
+  
+  if (hilt == 0)
+    hilt++;
+}
+
 unsigned
 apoint_inday (apoint_llist_node_t *i, long start)
 {
@@ -302,24 +413,20 @@ apoint_sec2str (apoint_llist_node_t *o, int type, long day, char *start,
   time_t t;
 
   if (o->start < day && type == APPT)
-    {
-      strncpy (start, "..:..", 6);
-    }
+    (void)strncpy (start, "..:..", 6);
   else
     {
       t = o->start;
       lt = localtime (&t);
-      snprintf (start, HRMIN_SIZE, "%02u:%02u", lt->tm_hour, lt->tm_min);
+      (void)snprintf (start, HRMIN_SIZE, "%02u:%02u", lt->tm_hour, lt->tm_min);
     }
   if (o->start + o->dur > day + DAYINSEC && type == APPT)
-    {
-      strncpy (end, "..:..", 6);
-    }
+    (void)strncpy (end, "..:..", 6);
   else
     {
       t = o->start + o->dur;
       lt = localtime (&t);
-      snprintf (end, HRMIN_SIZE, "%02u:%02u", lt->tm_hour, lt->tm_min);
+      (void)snprintf (end, HRMIN_SIZE, "%02u:%02u", lt->tm_hour, lt->tm_min);
     }
 }
 
@@ -331,44 +438,41 @@ apoint_write (apoint_llist_node_t *o, FILE *f)
 
   t = o->start;
   lt = localtime (&t);
-  fprintf (f, "%02u/%02u/%04u @ %02u:%02u",
-	   lt->tm_mon + 1, lt->tm_mday, 1900 + lt->tm_year, lt->tm_hour,
-           lt->tm_min);
+  (void)fprintf (f, "%02u/%02u/%04u @ %02u:%02u",
+                 lt->tm_mon + 1, lt->tm_mday, 1900 + lt->tm_year, lt->tm_hour,
+                 lt->tm_min);
 
   t = o->start + o->dur;
   lt = localtime (&t);
-  fprintf (f, " -> %02u/%02u/%04u @ %02u:%02u ",
-	   lt->tm_mon + 1, lt->tm_mday, 1900 + lt->tm_year, lt->tm_hour,
-           lt->tm_min);
+  (void)fprintf (f, " -> %02u/%02u/%04u @ %02u:%02u ",
+                 lt->tm_mon + 1, lt->tm_mday, 1900 + lt->tm_year, lt->tm_hour,
+                 lt->tm_min);
 
   if (o->note != NULL)
-    fprintf (f, ">%s ", o->note);
+    (void)fprintf (f, ">%s ", o->note);
 
   if (o->state & APOINT_NOTIFY)
-    fprintf (f, "!");
+    (void)fprintf (f, "!");
   else
-    fprintf (f, "|");
+    (void)fprintf (f, "|");
 
-  fprintf (f, "%s\n", o->mesg);
+  (void)fprintf (f, "%s\n", o->mesg);
 }
 
 apoint_llist_node_t *
 apoint_scan (FILE *f, struct tm start, struct tm end, char state, char *note)
 {
-  struct tm *lt;
-  char buf[MESG_MAXSIZE], *nl;
+  char buf[MESG_MAXSIZE], *newline;
   time_t tstart, tend, t;
 
   t = time (NULL);
-  lt = localtime (&t);
+  (void)localtime (&t);
 
   /* Read the appointment description */
-  fgets (buf, MESG_MAXSIZE, f);
-  nl = strchr (buf, '\n');
-  if (nl)
-    {
-      *nl = '\0';
-    }
+  (void)fgets (buf, MESG_MAXSIZE, f);
+  newline = strchr (buf, '\n');
+  if (newline)
+    *newline = '\0';
 
   start.tm_sec = end.tm_sec = 0;
   start.tm_isdst = end.tm_isdst = -1;
@@ -422,22 +526,30 @@ apoint_delete_bynum (long start, unsigned num, erase_flag_e flag)
 	{
 	  if (n == num)
 	    {
-	      if (flag == ERASE_FORCE_ONLY_NOTE)
+              switch (flag)
                 {
+                case ERASE_FORCE_ONLY_NOTE:
                   erase_note (&i->note, flag);
                   pthread_mutex_unlock (&(alist_p->mutex));
-                }
-	      else
-		{
+                  break;
+                case ERASE_CUT:
+                  apoint_free_bkp (ERASE_FORCE);
+                  apoint_dup (i, &bkp_cut_apoint);
+                  if (i->note)
+                    mem_free (i->note);
+                  /* FALLTHROUGH */
+                default:
 		  if (notify_bar ())
 		    need_check_notify = notify_same_item (i->start);
 		  *iptr = i->next;
-		  free (i->mesg);
-		  erase_note (&i->note, flag);
-		  free (i);
+		  mem_free (i->mesg);
+                  if (flag != ERASE_FORCE_KEEP_NOTE && flag != ERASE_CUT)
+                    erase_note (&i->note, flag);
+		  mem_free (i);
 		  pthread_mutex_unlock (&(alist_p->mutex));
 		  if (need_check_notify)
 		    notify_check_next_app ();
+                  break;
 		}
 	      return;
 	    }
@@ -445,10 +557,10 @@ apoint_delete_bynum (long start, unsigned num, erase_flag_e flag)
 	}
       iptr = &i->next;
     }
+  
   pthread_mutex_unlock (&(alist_p->mutex));
-
-  /* NOTREACHED */
   EXIT (_("no such appointment"));
+  /* NOTREACHED */
 }
 
 /*
@@ -487,9 +599,9 @@ apoint_scroll_pad_down (int nb_events_inday, int win_length)
     item_last_line = item_first_line;
   else
     item_last_line = item_first_line + 1;
-  pad_last_line = apad->first_onscreen + awin_length;
+  pad_last_line = apad.first_onscreen + awin_length;
   if (item_last_line >= pad_last_line)
-    apad->first_onscreen = item_last_line - awin_length;
+    apad.first_onscreen = item_last_line - awin_length;
 }
 
 /* 
@@ -502,8 +614,8 @@ apoint_scroll_pad_up (int nb_events_inday)
   int item_first_line = 0;
 
   item_first_line = get_item_line (hilt, nb_events_inday);
-  if (item_first_line < apad->first_onscreen)
-    apad->first_onscreen = item_first_line;
+  if (item_first_line < apad.first_onscreen)
+    apad.first_onscreen = item_first_line;
 }
 
 /*
@@ -528,7 +640,7 @@ apoint_check_next (struct notify_app_s *app, long start)
 	  if (i->start > start)
 	    {
 	      app->time = i->start;
-	      app->txt = strdup (i->mesg);
+	      app->txt = mem_strdup (i->mesg);
 	      app->state = i->state;
 	      app->got_app = 1;
 	    }
@@ -548,11 +660,10 @@ apoint_recur_s2apoint_s (recur_apoint_llist_node_t *p)
 {
   apoint_llist_node_t *a;
 
-  a = (apoint_llist_node_t *) malloc (sizeof (apoint_llist_node_t));
-  a->mesg = (char *) malloc (strlen (p->mesg) + 1);
+  a = (apoint_llist_node_t *) mem_malloc (sizeof (apoint_llist_node_t));
+  a->mesg = mem_strdup (p->mesg);
   a->start = p->start;
   a->dur = p->dur;
-  a->mesg = p->mesg;
   return (a);
 }
 
@@ -605,10 +716,10 @@ apoint_switch_notify (void)
 	  n++;
 	}
     }
-  pthread_mutex_unlock (&(alist_p->mutex));
 
-  /* NOTREACHED */
+  pthread_mutex_unlock (&(alist_p->mutex));  
   EXIT (_("no such appointment"));
+  /* NOTREACHED */  
 }
 
 /* Updates the Appointment panel */
@@ -641,11 +752,11 @@ apoint_update_panel (int which_pan)
   custom_remove_attr (win[APP].p, ATTR_HIGHEST);
 
   /* Draw the scrollbar if necessary. */
-  if ((apad->length >= app_length) || (apad->first_onscreen > 0))
+  if ((apad.length >= app_length) || (apad.first_onscreen > 0))
     {
-      float ratio = ((float) app_length) / ((float) apad->length);
+      float ratio = ((float) app_length) / ((float) apad.length);
       int sbar_length = (int) (ratio * app_length);
-      int highend = (int) (ratio * apad->first_onscreen);
+      int highend = (int) (ratio * apad.first_onscreen);
       bool hilt_bar = (which_pan == APP) ? true : false;
       int sbar_top = highend + title_lines + 1;
 
@@ -656,8 +767,21 @@ apoint_update_panel (int which_pan)
     }
 
   wnoutrefresh (win[APP].p);
-  pnoutrefresh (apad->ptrwin, apad->first_onscreen, 0,
+  pnoutrefresh (apad.ptrwin, apad.first_onscreen, 0,
 		win[APP].y + title_lines + 1, win[APP].x + bordr,
 		win[APP].y + win[APP].h - 2 * bordr,
 		win[APP].x + win[APP].w - 3 * bordr);
 }
+
+void
+apoint_paste_item (void)
+{
+  long bkp_time, bkp_start;
+
+  bkp_time = get_item_time (bkp_cut_apoint.start);
+  bkp_start = calendar_get_slctd_day_sec () + bkp_time;
+  (void)apoint_new (bkp_cut_apoint.mesg, bkp_cut_apoint.note,
+                    bkp_start, bkp_cut_apoint.dur,
+                    bkp_cut_apoint.state);
+  apoint_free_bkp (ERASE_FORCE_KEEP_NOTE);
+}
