@@ -1,8 +1,8 @@
-/*	$calcurse: args.c,v 1.41 2008/12/07 09:20:38 culot Exp $	*/
+/*	$calcurse: args.c,v 1.45 2009/01/24 18:45:35 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2008 Frederic Culot
+ * Copyright (c) 2004-2009 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,8 +39,13 @@
 #include "apoint.h"
 #include "day.h"
 #include "todo.h"
+#include "mem.h"
 #include "io.h"
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 /* 
  * Print Calcurse usage and exit.
  */
@@ -72,7 +77,8 @@ version_arg ()
       _("\nCopyright (c) 2004-2008 Frederic Culot.\n"
 	"This is free software; see the source for copying conditions.\n");
 
-  snprintf (vtitle, BUFSIZ, _("Calcurse %s - text-based organizer\n"), VERSION);
+  (void)snprintf (vtitle, BUFSIZ, _("Calcurse %s - text-based organizer\n"),
+                  VERSION);
   fputs (vtitle, stdout);
   fputs (vtext, stdout);
 }
@@ -134,7 +140,8 @@ help_arg ()
       "or read the manpage.\n"
       "Mail bug reports and suggestions to <calcurse@culot.org>.\n");
 
-  snprintf (htitle, BUFSIZ, _("Calcurse %s - text-based organizer\n"), VERSION);
+  (void)snprintf (htitle, BUFSIZ, _("Calcurse %s - text-based organizer\n"),
+                  VERSION);
   fputs (htitle, stdout);
   usage ();
   fputs (htext, stdout);
@@ -159,13 +166,13 @@ print_notefile (FILE *out, char *filename, int nbtab)
   int printlinestarter = 1;
 
   for (i = 0; i < nbtab; i++)
-    snprintf(linestarter, BUFSIZ, "%s\t", linestarter);
+    (void)snprintf(linestarter, BUFSIZ, "%s\t", linestarter);
 
-  snprintf (path_to_notefile, BUFSIZ, "%s/%s", path_notes, filename);
+  (void)snprintf (path_to_notefile, BUFSIZ, "%s/%s", path_notes, filename);
   notefile = fopen (path_to_notefile, "r");
   if (notefile)
     {
-      while (fgets (buffer, BUFSIZ, notefile) != NULL)
+      while (fgets (buffer, BUFSIZ, notefile) != 0)
 	{
 	  if (printlinestarter)
 	    {
@@ -177,7 +184,7 @@ print_notefile (FILE *out, char *filename, int nbtab)
 	    printlinestarter = 1;
 	}
       fputs ("\n", out);
-      fclose (notefile);
+      file_close (notefile, __FILE_POS__);
     }
   else
     {
@@ -207,7 +214,7 @@ todo_arg (int priority, int print_note)
 	      fputs (_("to do:\n"), stdout);
 	      title = 0;
 	    }
-	  snprintf (priority_str, BUFSIZ, "%d. ", i->id);
+	  (void)snprintf (priority_str, BUFSIZ, "%d. ", i->id);
 	  fputs (priority_str, stdout);
 	  fputs (i->mesg, stdout);
 	  fputs ("\n", stdout);
@@ -239,10 +246,10 @@ next_arg (void)
       hours_left = (time_left / HOURINSEC);
       min_left = (time_left - hours_left * HOURINSEC) / MININSEC;
       fputs (_("next appointment:\n"), stdout);
-      snprintf (mesg, BUFSIZ, "   [%02d:%02d] %s\n", hours_left, min_left,
-                next_app.txt);
+      (void)snprintf (mesg, BUFSIZ, "   [%02d:%02d] %s\n", hours_left, min_left,
+                      next_app.txt);
       fputs (mesg, stdout);
-      free (next_app.txt);
+      mem_free (next_app.txt);
     }
 }
 
@@ -434,7 +441,7 @@ display_app (struct tm *t, int numdays, int add_line, int print_note,
       if (app_found)
         add_line = 1;
       t->tm_mday++;
-      mktime (t);
+      (void)mktime (t);
     }
 }
 
@@ -487,9 +494,9 @@ date_arg (char *ddate, int add_line, int print_note, conf_t *conf)
 	{
 	  char outstr[BUFSIZ];
 	  fputs (_("Argument to the '-d' flag is not valid\n"), stderr);
-	  snprintf (outstr, BUFSIZ,
-		    "Possible argument format are: '%s' or 'n'\n",
-		    DATEFMT_DESC (conf->input_datefmt));
+	  (void)snprintf (outstr, BUFSIZ,
+                          "Possible argument format are: '%s' or 'n'\n",
+                          DATEFMT_DESC (conf->input_datefmt));
 	  fputs (_(outstr), stdout);
           more_info ();
 	}
@@ -535,7 +542,7 @@ date_arg_extended (char *startday, char *range, int add_line, int print_note,
 	{
 	  t.tm_year -= 1900;
 	  t.tm_mon--;
-	  mktime (&t);
+	  (void)mktime (&t);
 	}
       else
 	{
@@ -550,9 +557,9 @@ date_arg_extended (char *startday, char *range, int add_line, int print_note,
     {
       char outstr[BUFSIZ];
       fputs (_("Argument is not valid\n"), stderr);
-      snprintf (outstr, BUFSIZ,
-		"Argument format for -s and --startday is: '%s'\n",
-		DATEFMT_DESC (conf->input_datefmt));
+      (void)snprintf (outstr, BUFSIZ,
+                      "Argument format for -s and --startday is: '%s'\n",
+                      DATEFMT_DESC (conf->input_datefmt));
       fputs (_(outstr), stdout);
       fputs (_("Argument format for -r and --range is: 'n'\n"), stdout);
       more_info ();
@@ -723,7 +730,7 @@ parse_args (int argc, char **argv, conf_t *conf)
     {
       usage ();
       usage_try ();
-      return (EXIT_FAILURE);
+      return EXIT_FAILURE;
       /* Incorrect arguments */
     }
   else if (Dflag && cflag)
@@ -732,7 +739,7 @@ parse_args (int argc, char **argv, conf_t *conf)
                stderr);
       usage ();
       usage_try ();
-      return (EXIT_FAILURE);
+      return EXIT_FAILURE;
     }
   else
     {
@@ -757,26 +764,27 @@ parse_args (int argc, char **argv, conf_t *conf)
 	      io_init (cfile, datadir);
 	      no_file = io_check_data_files ();
 	      if (dflag || aflag || nflag || iflag || xflag || rflag || sflag)
-		io_load_app ();
+                {
+                  vars_init (conf);
+                  custom_load_conf (conf, 0);
+                  io_load_keys (conf->pager);
+                  io_load_app ();
+                  io_load_todo ();
+                }
 	    }
           if (iflag)
             {
               notify_init_vars ();
-	      vars_init (conf);              
-              custom_load_conf (conf, 0);
-              io_load_todo ();
               io_import_data (IO_IMPORT_ICAL, conf, ifile);
-              io_save_cal (conf);
+              io_save_cal (conf, IO_SAVE_DISPLAY_NONE);
               non_interactive = 1;
             }
 	  if (xflag)
 	    {
 	      notify_init_vars ();
-	      custom_load_conf (conf, 0);
-              io_load_todo ();
 	      io_export_data (xfmt, conf);
 	      non_interactive = 1;
-	      return (non_interactive);
+	      return non_interactive;
 	    }
 	  if (tflag)
 	    {
@@ -791,8 +799,6 @@ parse_args (int argc, char **argv, conf_t *conf)
 	  if (dflag || rflag || sflag)
 	    {
 	      notify_init_vars ();
-	      vars_init (conf);
-	      custom_load_conf (conf, 0);
               if (dflag)
                 date_arg (ddate, add_line, Nflag, conf);
               if (rflag || sflag)
@@ -804,8 +810,6 @@ parse_args (int argc, char **argv, conf_t *conf)
 	      date_t day;
 	      day.dd = day.mm = day.yyyy = 0;
 	      notify_init_vars ();
-	      vars_init (conf);
-	      custom_load_conf (conf, 0);
 	      app_found = app_arg (add_line, &day, 0, Nflag, conf);
 	      non_interactive = 1;
 	    }
@@ -816,6 +820,6 @@ parse_args (int argc, char **argv, conf_t *conf)
 	  io_init (cfile, datadir);
 	  no_file = io_check_data_files ();
 	}
-      return (non_interactive);
     }
+  return non_interactive;  
 }
