@@ -1,8 +1,8 @@
-/*	$calcurse: calcurse.c,v 1.73 2008/12/20 19:27:31 culot Exp $	*/
+/*	$calcurse: calcurse.c,v 1.78 2009/01/24 18:45:35 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2008 Frederic Culot
+ * Copyright (c) 2004-2009 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,13 +24,13 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif /* HAVE_CONFIG_H */
-
 #include <stdlib.h>
 #include <signal.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include "i18n.h"
 #include "io.h"
 #include "help.h"
@@ -60,6 +60,7 @@ main (int argc, char **argv)
   int no_data_file = 1;
   int sav_hilt_app = 0;
   int sav_hilt_tod = 0;
+  int cut_item = 0;
   struct sigaction sigact;
   bool do_storage = false;
   bool do_update = true;
@@ -86,7 +87,7 @@ main (int argc, char **argv)
    */
   non_interactive = parse_args (argc, argv, &conf);
   if (non_interactive)
-    return (EXIT_SUCCESS);
+    exit_calcurse (EXIT_SUCCESS);
 
   /* Begin of interactive mode with ncurses interface. */
   sigs_init (&sigact);		/* signal handling init */
@@ -133,11 +134,10 @@ main (int argc, char **argv)
     }
 
   vars_init (&conf);
-  keys_init ();
   wins_init ();
   wins_slctd_init ();
   notify_init_bar ();
-  reset_status_page ();
+  wins_reset_status_page ();
 
   /* 
    * Read the data from files : first the user
@@ -146,7 +146,7 @@ main (int argc, char **argv)
    */
   no_data_file = io_check_data_files ();
   custom_load_conf (&conf, background);
-  erase_status_bar ();
+  wins_erase_status_bar ();
   io_load_keys (conf.pager);
   io_load_todo ();
   io_load_app ();
@@ -159,7 +159,9 @@ main (int argc, char **argv)
   wins_slctd_set (CAL);
   wins_update ();
   calendar_start_date_thread ();
-
+  if (conf.periodic_save > 0)
+    io_start_psave_thread (&conf);
+  
   /* User input */
   for (;;)
     {
@@ -180,7 +182,7 @@ main (int argc, char **argv)
 	  break;
 
 	case KEY_GENERIC_CHANGE_VIEW:
-	  reset_status_page ();
+	  wins_reset_status_page ();
 	  /* Need to save the previously highlighted event. */
 	  switch (wins_slctd ())
 	    {
@@ -219,12 +221,12 @@ main (int argc, char **argv)
 	  break;
 
         case KEY_GENERIC_OTHER_CMD:
-	  other_status_page (wins_slctd ());
+	  wins_other_status_page (wins_slctd ());
 	  break;
 
         case KEY_GENERIC_GOTO:
         case KEY_GENERIC_GOTO_TODAY:
-	  erase_status_bar ();
+	  wins_erase_status_bar ();
 	  calendar_set_current_date ();
           if (key == KEY_GENERIC_GOTO_TODAY)
             calendar_goto_today ();
@@ -242,7 +244,7 @@ main (int argc, char **argv)
 	  break;
 
         case KEY_GENERIC_CONFIG_MENU:
-	  erase_status_bar ();
+	  wins_erase_status_bar ();
 	  custom_config_bar ();
 	  while ((key = wgetch (win[STA].p)) != 'q')
 	    {
@@ -255,7 +257,7 @@ main (int argc, char **argv)
 		  else
 		    {
 		      colorize = false;
-		      erase_status_bar ();
+		      wins_erase_status_bar ();
 		      mvwprintw (win[STA].p, 0, 0, _(no_color_support));
 		      wgetch (win[STA].p);
 		    }
@@ -280,7 +282,7 @@ main (int argc, char **argv)
 	      wins_reset ();
 	      wins_update ();
 	      do_storage = true;
-	      erase_status_bar ();
+	      wins_erase_status_bar ();
 	      custom_config_bar ();
 	    }
 	  wins_update ();
@@ -330,6 +332,23 @@ main (int argc, char **argv)
 	  do_storage = true;
 	  break;
 
+        case KEY_GENERIC_CUT:
+          if (wins_slctd () == APP && apoint_hilt () != 0)
+            {
+              cut_item = apoint_cut (&inday.nb_events, &inday.nb_apoints);
+              do_storage = true;
+            }
+          break;
+
+        case KEY_GENERIC_PASTE:
+          if (wins_slctd () == APP)
+            {
+              apoint_paste (&inday.nb_events, &inday.nb_apoints, cut_item);
+              cut_item = 0;
+              do_storage = true;
+            }
+          break;
+          
         case KEY_REPEAT_ITEM:
 	  if (wins_slctd () == APP && apoint_hilt () != 0)
 	    recur_repeat_item (&conf);
@@ -370,22 +389,22 @@ main (int argc, char **argv)
 	  break;
 
         case KEY_GENERIC_HELP:
-	  status_bar ();
+	  wins_status_bar ();
 	  help_screen ();
 	  break;
 
         case KEY_GENERIC_SAVE:
-	  io_save_cal (&conf);
+	  io_save_cal (&conf, IO_SAVE_DISPLAY_BAR);
 	  break;
 
         case KEY_GENERIC_IMPORT:
-          erase_status_bar ();
+          wins_erase_status_bar ();
           io_import_data (IO_IMPORT_ICAL, &conf, NULL);
           do_storage = true;
           break;
           
         case KEY_GENERIC_EXPORT:
-          erase_status_bar ();
+          wins_erase_status_bar ();
           io_export_bar ();
           while ((key = wgetch (win[STA].p)) != 'q')
 	    {
@@ -403,7 +422,7 @@ main (int argc, char **argv)
 	      wins_reset ();
 	      wins_update ();
 	      do_storage = true;
-	      erase_status_bar ();
+	      wins_erase_status_bar ();
 	      io_export_bar ();
 	    }
 	  wins_update ();
@@ -492,7 +511,7 @@ main (int argc, char **argv)
           
         case KEY_GENERIC_QUIT:          
 	  if (conf.auto_save)
-	    io_save_cal (&conf);
+	    io_save_cal (&conf, IO_SAVE_DISPLAY_BAR);
 
 	  if (conf.confirm_quit)
 	    {
@@ -502,7 +521,7 @@ main (int argc, char **argv)
 		exit_calcurse (EXIT_SUCCESS);
 	      else
 		{
-		  erase_status_bar ();
+		  wins_erase_status_bar ();
 		  break;
 		}
 	    }
