@@ -1,8 +1,8 @@
-/*	$calcurse: calendar.c,v 1.20 2008/12/14 15:54:51 culot Exp $	*/
+/*	$calcurse: calendar.c,v 1.22 2009/01/24 20:41:52 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2008 Frederic Culot
+ * Copyright (c) 2004-2009 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -63,25 +63,22 @@ static pthread_t calendar_t_date;
 
 
 /* Thread needed to update current date in calendar. */
+/* ARGSUSED0 */
 static void *
 calendar_date_thread (void *arg)
 {
-  time_t now, tomorrow;
+  time_t actual, tomorrow;
 
   for (;;)
     {
       tomorrow = (time_t) (get_today () + DAYINSEC);
 
-      while ((now = time (NULL)) < tomorrow)
-	{
-	  sleep (tomorrow - now);
-	}
+      while ((actual = time (NULL)) < tomorrow)
+        (void)sleep (tomorrow - actual);
 
       calendar_set_current_date ();
       calendar_update_panel (win[CAL].p);
     }
-
-  pthread_exit ((void *) 0);
 }
 
 /* Launch the calendar date thread. */
@@ -89,15 +86,14 @@ void
 calendar_start_date_thread (void)
 {
   pthread_create (&calendar_t_date, NULL, calendar_date_thread, NULL);
-  return;
 }
 
 /* Stop the calendar date thread. */
 void
 calendar_stop_date_thread (void)
 {
-  pthread_cancel (calendar_t_date);
-  return;
+  if (calendar_t_date)
+    pthread_cancel (calendar_t_date);
 }
 
 /* Set static variable today to current date */
@@ -362,7 +358,7 @@ calendar_change_day (int datefmt)
 
   while (wrong_day)
     {
-      snprintf (outstr, BUFSIZ, request_date, DATEFMT_DESC (datefmt));
+      (void)snprintf (outstr, BUFSIZ, request_date, DATEFMT_DESC (datefmt));
       status_mesg (_(outstr), "");
       if (getstring (win[STA].p, selected_day, LDAY, 0, 1) == GETSTRING_ESC)
 	return;
@@ -424,7 +420,7 @@ calendar_move (move_t move)
   int ret, days_to_remove, days_to_add;
   struct tm t;
 
-  memset (&t, 0, sizeof (struct tm));
+  (void)memset (&t, 0, sizeof (struct tm));
   t.tm_mday = slctd_day.dd;
   t.tm_mon = slctd_day.mm - 1;
   t.tm_year = slctd_day.yyyy - 1900;
@@ -458,7 +454,7 @@ calendar_move (move_t move)
       break;
     case WEEK_START:
       /* Normalize struct tm to get week day number. */
-      mktime (&t);
+      (void)mktime (&t);
       if (calendar_week_begins_on_monday ())
         days_to_remove = ((t.tm_wday == 0) ? WEEKINDAYS - 1 : t.tm_wday - 1);
       else
@@ -466,7 +462,7 @@ calendar_move (move_t move)
       ret = date_change (&t, 0, 0 - days_to_remove);
       break;
     case WEEK_END:
-      mktime (&t);
+      (void)mktime (&t);
       if (calendar_week_begins_on_monday ())
         days_to_add = ((t.tm_wday == 0) ? 0 : WEEKINDAYS - t.tm_wday);
       else
