@@ -1,8 +1,8 @@
-/*	$calcurse: custom.c,v 1.31 2008/12/20 19:27:31 culot Exp $	*/
+/*	$calcurse: custom.c,v 1.37 2009/01/24 14:44:25 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2008 Frederic Culot
+ * Copyright (c) 2004-2009 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,13 +28,18 @@
 #include <stdlib.h>
 #include <math.h>
 
-#include "custom.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include "i18n.h"
 #include "io.h"
 #include "utils.h"
 #include "keys.h"
 #include "apoint.h"
 #include "help.h"
+#include "mem.h"
+#include "custom.h"
 
 static struct attribute_s attr;
 
@@ -214,10 +219,10 @@ custom_load_conf (conf_t *conf, int background)
       status_mesg (mesg_line1, mesg_line2);
       wnoutrefresh (win[STA].p);
       doupdate ();
-      keys_getch (win[STA].p);
+      (void)keys_getch (win[STA].p);
     }
   var = CUSTOM_CONF_NOVARIABLE;
-  pthread_mutex_lock (&nbar->mutex);
+  pthread_mutex_lock (&nbar.mutex);
   for (;;)
     {
       if (fgets (buf, 99, data_file) == NULL)
@@ -234,6 +239,12 @@ custom_load_conf (conf_t *conf, int background)
 	  conf->auto_save = fill_config_var (e_conf);
 	  var = 0;
 	  break;
+	case CUSTOM_CONF_PERIODICSAVE:
+	  conf->periodic_save = atoi (e_conf);
+	  if (conf->periodic_save < 0)
+	    conf->periodic_save = 0;
+	  var = 0;
+	  break;
 	case CUSTOM_CONF_CONFIRMQUIT:
 	  conf->confirm_quit = fill_config_var (e_conf);
 	  var = 0;
@@ -266,33 +277,33 @@ custom_load_conf (conf_t *conf, int background)
 	  var = 0;
 	  break;
 	case CUSTOM_CONF_NOTIFYBARSHOW:
-	  nbar->show = fill_config_var (e_conf);
+	  nbar.show = fill_config_var (e_conf);
 	  var = 0;
 	  break;
 	case CUSTOM_CONF_NOTIFYBARDATE:
-	  strncpy (nbar->datefmt, e_conf, strlen (e_conf) + 1);
+	  (void)strncpy (nbar.datefmt, e_conf, strlen (e_conf) + 1);
 	  var = 0;
 	  break;
 	case CUSTOM_CONF_NOTIFYBARCLOCK:
-	  strncpy (nbar->timefmt, e_conf, strlen (e_conf) + 1);
+	  (void)strncpy (nbar.timefmt, e_conf, strlen (e_conf) + 1);
 	  var = 0;
 	  break;
 	case CUSTOM_CONF_NOTIFYBARWARNING:
-	  nbar->cntdwn = atoi (e_conf);
+	  nbar.cntdwn = atoi (e_conf);
 	  var = 0;
 	  break;
 	case CUSTOM_CONF_NOTIFYBARCOMMAND:
-	  strncpy (nbar->cmd, e_conf, strlen (e_conf) + 1);
+	  (void)strncpy (nbar.cmd, e_conf, strlen (e_conf) + 1);
 	  var = 0;
 	  break;
 	case CUSTOM_CONF_OUTPUTDATEFMT:
 	  if (e_conf[0] != '\0')
-	    strncpy (conf->output_datefmt, e_conf, strlen (e_conf) + 1);
+	    (void)strncpy (conf->output_datefmt, e_conf, strlen (e_conf) + 1);
 	  var = 0;
 	  break;
 	case CUSTOM_CONF_INPUTDATEFMT:
 	  conf->input_datefmt = atoi (e_conf);
-	  if (conf->input_datefmt < 1 || conf->input_datefmt > 3)
+	  if (conf->input_datefmt <= 0 || conf->input_datefmt >= DATE_FORMATS)
 	    conf->input_datefmt = 1;
 	  var = 0;
 	  break;
@@ -303,6 +314,8 @@ custom_load_conf (conf_t *conf, int background)
 
       if (strncmp (e_conf, "auto_save=", 10) == 0)
 	var = CUSTOM_CONF_AUTOSAVE;
+      else if (strncmp (e_conf, "periodic_save=", 14) == 0)
+        var = CUSTOM_CONF_PERIODICSAVE;
       else if (strncmp (e_conf, "confirm_quit=", 13) == 0)
 	var = CUSTOM_CONF_CONFIRMQUIT;
       else if (strncmp (e_conf, "confirm_delete=", 15) == 0)
@@ -332,8 +345,8 @@ custom_load_conf (conf_t *conf, int background)
       else if (strncmp (e_conf, "input_datefmt=", 12) == 0)
 	var = CUSTOM_CONF_INPUTDATEFMT;
     }
-  fclose (data_file);
-  pthread_mutex_unlock (&nbar->mutex);
+  file_close (data_file, __FILE_POS__);
+  pthread_mutex_unlock (&nbar.mutex);
 }
 
 /* Draws the configuration bar */
@@ -423,10 +436,8 @@ display_layout_config (window_t *lwin, int mark, int cursor, int need_reset)
     {
       char label[BUFSIZ];
       
-      if (lwin->p != NULL)
-	delwin (lwin->p);
-      snprintf (label, BUFSIZ, _("CalCurse %s | layout configuration"),
-                VERSION);
+      (void)snprintf (label, BUFSIZ, _("CalCurse %s | layout configuration"),
+                      VERSION);
       custom_confwin_init (lwin, label);
     }
 
@@ -538,6 +549,13 @@ custom_layout_config (void)
 void
 custom_confwin_init (window_t *confwin, char *label)
 {
+  if (confwin->p)
+    {
+      erase_window_part (confwin->p, confwin->x, confwin->y,
+                         confwin->x + confwin->w, confwin->y + confwin->h);
+      (void)delwin (confwin->p);
+    }
+  
   wins_get_config ();
   confwin->h = (notify_bar ())? row - 3 : row - 2;
   confwin->p = newwin (confwin->h, col, 0, 0);
@@ -623,9 +641,7 @@ display_color_config (window_t *cwin, int *mark_fore, int *mark_back,
 
   if (need_reset)
     {
-      if (cwin->p != NULL)
-	delwin (cwin->p);
-      snprintf (label, BUFSIZ, _("CalCurse %s | color theme"), VERSION);
+      (void)snprintf (label, BUFSIZ, _("CalCurse %s | color theme"), VERSION);
       custom_confwin_init (cwin, label);
     }
 
@@ -718,7 +734,7 @@ custom_color_config (void)
   cursor = 0;
   need_reset = 1;
   theme_changed = 0;
-  conf_win.p = NULL;
+  conf_win.p = 0;
   display_color_config (&conf_win, &mark_fore, &mark_back, cursor,
 			need_reset, theme_changed);
 
@@ -809,7 +825,7 @@ custom_color_theme_name (char *theme_name)
   };
 
   if (!colorize)
-    snprintf (theme_name, BUFSIZ, "0");
+    (void)snprintf (theme_name, BUFSIZ, "0");
   else
     {
       pair_content (COLR_CUSTOM, &color[0], &color[1]);
@@ -825,7 +841,8 @@ custom_color_theme_name (char *theme_name)
 	      /* NOTREACHED */
 	    }
 	}
-      snprintf (theme_name, BUFSIZ, "%s on %s", color_name[0], color_name[1]);
+      (void)snprintf (theme_name, BUFSIZ, "%s on %s", color_name[0],
+                      color_name[1]);
     }
 }
 
@@ -833,72 +850,99 @@ custom_color_theme_name (char *theme_name)
 static int
 print_general_options (WINDOW *win, conf_t *conf)
 {
+  enum {
+    AUTO_SAVE,
+    PERIODIC_SAVE,
+    CONFIRM_QUIT,
+    CONFIRM_DELETE,
+    SKIP_SYSTEM_DIAGS,
+    SKIP_PROGRESS_BAR,
+    WEEK_BEGINS_MONDAY,
+    OUTPUT_DATE_FMT,
+    INPUT_DATE_FMT,
+    NB_OPTIONS
+  };
   const int XPOS = 1;
   const int YOFF = 3;
   int y;
-  char *opt1 = _("auto_save = ");
-  char *opt2 = _("confirm_quit = ");
-  char *opt3 = _("confirm_delete = ");
-  char *opt4 = _("skip_system_dialogs = ");
-  char *opt5 = _("skip_progress_bar = ");
-  char *opt6 = _("week_begins_on_monday = ");
-  char *opt7 = _("output_datefmt = ");
-  char *opt8 = _("input_datefmt = ");
+  char *opt[NB_OPTIONS] = {
+    _("auto_save = "),
+    _("periodic_save = "),    
+    _("confirm_quit = "),
+    _("confirm_delete = "),
+    _("skip_system_dialogs = "),
+    _("skip_progress_bar = "),
+    _("week_begins_on_monday = "),
+    _("output_datefmt = "),
+    _("input_datefmt = ")
+  };
   
   y = 0;
-  mvwprintw (win, y, XPOS, "[1] %s      ", opt1);
+  mvwprintw (win, y, XPOS, "[1] %s      ", opt[AUTO_SAVE]);
   print_bool_option_incolor (win, conf->auto_save, y,
-                             XPOS + 4 + strlen (opt1));
+                             XPOS + 4 + strlen (opt[AUTO_SAVE]));
   mvwprintw (win, y + 1, XPOS,
 	     _("(if set to YES, automatic save is done when quitting)"));
   y += YOFF;
-  mvwprintw (win, y, XPOS, "[2] %s      ", opt2);
+  mvwprintw (win, y, XPOS, "[2] %s      ", opt[PERIODIC_SAVE]);
+  custom_apply_attr (win, ATTR_HIGHEST);
+  mvwprintw (win, y, XPOS + 4 + strlen (opt[PERIODIC_SAVE]), "%d",
+             conf->periodic_save);
+  custom_remove_attr (win, ATTR_HIGHEST);
+  mvwprintw (win, y + 1, XPOS,
+	     _("(if not null, automatically save data every 'periodic_save' "
+               "minutes)"));
+  y += YOFF;
+  mvwprintw (win, y, XPOS, "[3] %s      ", opt[CONFIRM_QUIT]);
   print_bool_option_incolor (win, conf->confirm_quit, y,
-                             XPOS + 4 + strlen (opt2));
+                             XPOS + 4 + strlen (opt[CONFIRM_QUIT]));
   mvwprintw (win, y + 1, XPOS,
 	     _("(if set to YES, confirmation is required before quitting)"));
   y += YOFF;
-  mvwprintw (win, y, XPOS, "[3] %s      ", opt3);
+  mvwprintw (win, y, XPOS, "[4] %s      ", opt[CONFIRM_DELETE]);
   print_bool_option_incolor (win, conf->confirm_delete, y,
-                             XPOS + 4 + strlen (opt3));
+                             XPOS + 4 + strlen (opt[CONFIRM_DELETE]));
   mvwprintw (win, y + 1, XPOS,
 	     _("(if set to YES, confirmation is required "
                "before deleting an event)"));
   y += YOFF;
-  mvwprintw (win, y, XPOS, "[4] %s      ", opt4);
+  mvwprintw (win, y, XPOS, "[5] %s      ", opt[SKIP_SYSTEM_DIAGS]);
   print_bool_option_incolor (win, conf->skip_system_dialogs, y,
-                             XPOS + 4 + strlen (opt4));
+                             XPOS + 4 + strlen (opt[SKIP_SYSTEM_DIAGS]));
   mvwprintw (win, y + 1, XPOS,
 	     _("(if set to YES, messages about loaded "
                "and saved data will not be displayed)"));
   y += YOFF;
-  mvwprintw (win, y, XPOS, "[5] %s      ", opt5);
+  mvwprintw (win, y, XPOS, "[6] %s      ", opt[SKIP_PROGRESS_BAR]);
   print_bool_option_incolor (win, conf->skip_progress_bar, y,
-                            XPOS + 4 + strlen (opt5));
+                            XPOS + 4 + strlen (opt[SKIP_PROGRESS_BAR]));
   mvwprintw (win, y + 1, XPOS,
 	     _("(if set to YES, progress bar will not be displayed "
                 "when saving data)"));
   y += YOFF;
-  mvwprintw (win, y, XPOS, "[6] %s      ", opt6);
+  mvwprintw (win, y, XPOS, "[7] %s      ", opt[WEEK_BEGINS_MONDAY]);
   print_bool_option_incolor (win, calendar_week_begins_on_monday (), y,
-                             XPOS + 4 + strlen (opt6));
+                             XPOS + 4 + strlen (opt[WEEK_BEGINS_MONDAY]));
   mvwprintw (win, y + 1, XPOS,
 	     _("(if set to YES, monday is the first day of the week, "
                "else it is sunday)"));
   y += YOFF;
-  mvwprintw (win, y, XPOS, "[7] %s      ", opt7);
+  mvwprintw (win, y, XPOS, "[8] %s      ", opt[OUTPUT_DATE_FMT]);
   custom_apply_attr (win, ATTR_HIGHEST);
-  mvwprintw (win, y, XPOS + 4 + strlen (opt7), "%s", conf->output_datefmt);
+  mvwprintw (win, y, XPOS + 4 + strlen (opt[OUTPUT_DATE_FMT]), "%s",
+             conf->output_datefmt);
   custom_remove_attr (win, ATTR_HIGHEST);
   mvwprintw (win, y + 1, XPOS,
 	     _("(Format of the date to be displayed in non-interactive mode)"));
   y += YOFF;
-  mvwprintw (win, y, XPOS, "[8] %s      ", opt8);
+  mvwprintw (win, y, XPOS, "[9] %s      ", opt[INPUT_DATE_FMT]);
   custom_apply_attr (win, ATTR_HIGHEST);
-  mvwprintw (win, y, XPOS + 4 + strlen (opt8), "%d", conf->input_datefmt);
+  mvwprintw (win, y, XPOS + 4 + strlen (opt[INPUT_DATE_FMT]), "%d",
+             conf->input_datefmt);
   custom_remove_attr (win, ATTR_HIGHEST);
   mvwprintw (win, y + 1, XPOS, _("(Format to be used when entering a date: "));
-  mvwprintw (win, y + 2, XPOS, _(" 1-mm/dd/yyyy, 2-dd/mm/yyyy, 3-yyyy/mm/dd)"));
+  mvwprintw (win, y + 2, XPOS,
+             _(" (1)mm/dd/yyyy (2)dd/mm/yyyy (3)yyyy/mm/dd (4)yyyy-mm-dd)"));
 
   return y + YOFF;
 }
@@ -929,21 +973,28 @@ custom_general_config (conf_t *conf)
   char *output_datefmt_str =
     _("Enter the date format (see 'man 3 strftime' for possible formats) ");
   char *input_datefmt_str =
-    _("Enter the date format (1-mm/dd/yyyy, 2-dd/mm/yyyy, 3-yyyy/mm/dd) ");
+    _("Enter the date format (1)mm/dd/yyyy (2)dd/mm/yyyy (3)yyyy/mm/dd "
+      "(4)yyyy-mm-dd");
+  char *periodic_save_str =
+    _("Enter the delay, in minutes, between automatic saves (0 to disable) ");
   int ch;
-  char *buf = (char *) malloc (BUFSIZ);
+  char *buf;
 
   clear ();
   conf_set_scrsize (&cwin);
-  snprintf (cwin.label, BUFSIZ, _("CalCurse %s | general options"), VERSION);
+  (void)snprintf (cwin.label, BUFSIZ, _("CalCurse %s | general options"),
+                  VERSION);
   wins_scrollwin_init (&cwin);
   wins_show (cwin.win.p, cwin.label);
   status_mesg (number_str, keys);
   cwin.total_lines = print_general_options (cwin.pad.p, conf);
   wins_scrollwin_display (&cwin);
 
+  buf = mem_malloc (BUFSIZ);
   while ((ch = wgetch (win[STA].p)) != 'q')
     {
+      buf[0] = '\0';
+      
       switch (ch)
 	{
 	case KEY_RESIZE:
@@ -974,36 +1025,50 @@ custom_general_config (conf_t *conf)
 	  conf->auto_save = !conf->auto_save;
 	  break;
 	case '2':
-	  conf->confirm_quit = !conf->confirm_quit;
-	  break;
-	case '3':
-	  conf->confirm_delete = !conf->confirm_delete;
-	  break;
-	case '4':
-	  conf->skip_system_dialogs = !conf->skip_system_dialogs;
-	  break;
-	case '5':
-	  conf->skip_progress_bar = !conf->skip_progress_bar;
-	  break;
-	case '6':
-	  calendar_change_first_day_of_week ();
-	  break;
-	case '7':
-	  status_mesg (output_datefmt_str, "");
-	  strncpy (buf, conf->output_datefmt,
-		   strlen (conf->output_datefmt) + 1);
+	  status_mesg (periodic_save_str, "");
 	  if (updatestring (win[STA].p, &buf, 0, 1) == 0)
 	    {
-	      strncpy (conf->output_datefmt, buf, strlen (buf) + 1);
+	      int val = atoi (buf);
+	      if (val >= 0)
+		conf->periodic_save = val;
+              if (conf->periodic_save > 0)
+                io_start_psave_thread (conf);
+              else if (conf->periodic_save == 0)
+                io_stop_psave_thread ();
 	    }
 	  status_mesg (number_str, keys);
 	  break;
+	case '3':
+	  conf->confirm_quit = !conf->confirm_quit;
+	  break;
+	case '4':
+	  conf->confirm_delete = !conf->confirm_delete;
+	  break;
+	case '5':
+	  conf->skip_system_dialogs = !conf->skip_system_dialogs;
+	  break;
+	case '6':
+	  conf->skip_progress_bar = !conf->skip_progress_bar;
+	  break;
+	case '7':
+	  calendar_change_first_day_of_week ();
+	  break;
 	case '8':
+	  status_mesg (output_datefmt_str, "");
+	  (void)strncpy (buf, conf->output_datefmt,
+                         strlen (conf->output_datefmt) + 1);
+	  if (updatestring (win[STA].p, &buf, 0, 1) == 0)
+	    {
+	      (void)strncpy (conf->output_datefmt, buf, strlen (buf) + 1);
+	    }
+	  status_mesg (number_str, keys);
+	  break;
+	case '9':
 	  status_mesg (input_datefmt_str, "");
 	  if (updatestring (win[STA].p, &buf, 0, 1) == 0)
 	    {
 	      int val = atoi (buf);
-	      if (val >= 1 && val <= 3)
+	      if (val > 0 && val <= DATE_FORMATS)
 		conf->input_datefmt = val;
 	    }
 	  status_mesg (number_str, keys);
@@ -1013,7 +1078,7 @@ custom_general_config (conf_t *conf)
       cwin.total_lines = print_general_options (cwin.pad.p, conf);
       wins_scrollwin_display (&cwin);
     }
-  free (buf);
+  mem_free (buf);
   wins_scrollwin_delete (&cwin);
 }
 
@@ -1045,7 +1110,7 @@ print_keys_bindings (WINDOW *win, int selected_row, int selected_elm, int yoff)
       int nbkeys;
 
       nbkeys = keys_action_count_keys (action);      
-      snprintf (actionstr, BUFSIZ, "%s", keys_get_label (action));
+      (void)snprintf (actionstr, BUFSIZ, "%s", keys_get_label (action));
       if (action == selected_row)
         custom_apply_attr (win, ATTR_HIGHEST);
       mvwprintw (win, y, XPOS, "%s ", actionstr);
@@ -1108,7 +1173,7 @@ custom_keys_config (void)
 {
   scrollwin_t kwin;
   int selrow, selelm, firstrow, lastrow, nbrowelm, nbdisplayed;
-  int keyval, used;
+  int keyval, used, not_recognized;
   char *keystr;
   WINDOW *grabwin;
   const int LINESPERKEY = 2;
@@ -1117,7 +1182,8 @@ custom_keys_config (void)
   clear ();
   conf_set_scrsize (&kwin);
   nbdisplayed = (kwin.win.h - LABELLINES) / LINESPERKEY;
-  snprintf (kwin.label, BUFSIZ, _("CalCurse %s | keys configuration"), VERSION);
+  (void)snprintf (kwin.label, BUFSIZ, _("CalCurse %s | keys configuration"),
+                  VERSION);
   wins_scrollwin_init (&kwin);
   wins_show (kwin.win.p, kwin.label);
   custom_keys_config_bar ();
@@ -1176,20 +1242,36 @@ custom_keys_config (void)
 #define WINCOL 50
           do
             {
+              used = 0;
               grabwin = popup (WINROW, WINCOL, (row - WINROW) / 2,
                                (col - WINCOL) / 2,
                                _("Press the key you want to assign to:"),
                                keys_get_label (selrow), 0);
               keyval = wgetch (grabwin);
+
+              /* First check if this key would be recognized by calcurse. */
+              if (keys_str2int (keys_int2str (keyval)) == -1)
+                {
+                  not_recognized = 1;
+                  WARN_MSG (_("This key is not yet recognized by calcurse, "
+                              "please choose another one."));
+                  werase (kwin.pad.p);
+                  nbrowelm = print_keys_bindings (kwin.pad.p, selrow, selelm,
+                                                  LINESPERKEY);
+                  wins_scrollwin_display (&kwin);
+                  continue;
+                }
+              else
+                not_recognized = 0;
+
               used = keys_assign_binding (keyval, selrow);
               if (used)
                 {
                   keys_e action;
                   
                   action = keys_get_action (keyval);
-                  ERROR_MSG (
-                             _("This key is already in use for %s, "
-                               "please choose another one."),
+                  WARN_MSG (_("This key is already in use for %s, "
+                              "please choose another one."),
                              keys_get_label (action));
                   werase (kwin.pad.p);
                   nbrowelm = print_keys_bindings (kwin.pad.p, selrow, selelm,
@@ -1198,7 +1280,7 @@ custom_keys_config (void)
                 }
               delwin (grabwin);              
             }
-          while (used);
+          while (used || not_recognized);
           nbrowelm++;          
           if (selelm < nbrowelm - 1)
             selelm++;
@@ -1216,8 +1298,8 @@ custom_keys_config (void)
         case KEY_GENERIC_QUIT:
           if (keys_check_missing_bindings () != 0)
             {
-              ERROR_MSG (_("Some actions do not have any associated "
-                           "key bindings!"));              
+              WARN_MSG (_("Some actions do not have any associated "
+                          "key bindings!"));              
             }
           wins_scrollwin_delete (&kwin);
           return;
