@@ -1,8 +1,8 @@
-/*	$calcurse: day.c,v 1.42 2008/12/15 20:02:00 culot Exp $	*/
+/*	$calcurse: day.c,v 1.48 2009/01/03 21:32:11 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2008 Frederic Culot
+ * Copyright (c) 2004-2009 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,24 +35,30 @@
 #include "event.h"
 #include "custom.h"
 #include "keys.h"
+#include "mem.h"
 #include "day.h"
 
-static struct day_item_s *day_items_ptr;
-static struct day_saved_item_s *day_saved_item = NULL;
+static struct day_item_s        *day_items_ptr;
+static struct day_saved_item_s   day_saved_item;
 
-/* Free the current day linked list containing the events and appointments. */
-static void
+/*
+ * Free the current day linked list containing the events and appointments.
+ * Must not free associated message and note, because their are not dynamically
+ * allocated (only pointers to real objects are stored in this structure).
+ */
+void
 day_free_list (void)
 {
-  struct day_item_s *p, *q;
+  struct day_item_s *o, **i;
 
-  for (p = day_items_ptr; p != 0; p = q)
+  i = &day_items_ptr;
+  while (*i)
     {
-      q = p->next;
-      free (p->mesg);
-      free (p);
+      o = *i;
+      *i = o->next;
+      mem_free (o);
     }
-  day_items_ptr = NULL;
+  day_items_ptr = 0;
 }
 
 /* Add an event in the current day list */
@@ -60,9 +66,9 @@ static struct day_item_s *
 day_add_event (int type, char *mesg, char *note, long day, int id)
 {
   struct day_item_s *o, **i;
-  o = (struct day_item_s *) malloc (sizeof (struct day_item_s));
-  o->mesg = (char *) malloc (strlen (mesg) + 1);
-  strncpy (o->mesg, mesg, strlen (mesg) + 1);
+  
+  o = mem_malloc (sizeof (struct day_item_s));
+  o->mesg = mesg;
   o->note = note;
   o->type = type;
   o->appt_dur = 0;
@@ -91,8 +97,8 @@ day_add_apoint (int type, char *mesg, char *note, long start, long dur,
   struct day_item_s *o, **i;
   int insert_item = 0;
 
-  o = (struct day_item_s *) malloc (sizeof (struct day_item_s));
-  o->mesg = strdup (mesg);
+  o = mem_malloc (sizeof (struct day_item_s));
+  o->mesg = mesg;
   o->note = note;
   o->start = start;
   o->appt_dur = dur;
@@ -119,7 +125,7 @@ day_add_apoint (int type, char *mesg, char *note, long start, long dur,
 	}
       i = &(*i)->next;
     }
-  return (o);
+  return o;
 }
 
 /* 
@@ -145,7 +151,7 @@ day_store_events (long date)
 	}
     }
 
-  return (e_nb);
+  return e_nb;
 }
 
 /* 
@@ -172,7 +178,7 @@ day_store_recur_events (long date)
 	}
     }
 
-  return (e_nb);
+  return e_nb;
 }
 
 /* 
@@ -201,7 +207,7 @@ day_store_apoints (long date)
     }
   pthread_mutex_unlock (&(alist_p->mutex));
 
-  return (a_nb);
+  return a_nb;
 }
 
 /* 
@@ -234,7 +240,7 @@ day_store_recur_apoints (long date)
     }
   pthread_mutex_unlock (&(recur_alist_p->mutex));
 
-  return (a_nb);
+  return a_nb;
 }
 
 /* 
@@ -268,7 +274,7 @@ day_store_items (long date, unsigned *pnb_events, unsigned *pnb_apoints)
   *pnb_apoints += nb_recur_apoints;
   *pnb_events += nb_recur_events;
 
-  return (pad_length);
+  return pad_length;
 }
 
 /*
@@ -291,18 +297,18 @@ day_process_storage (date_t *slctd_date, bool day_changed,
   date = date2sec (day, 0, 0);
 
   /* Inits */
-  if (apad->length != 0)
-    delwin (apad->ptrwin);
+  if (apad.length != 0)
+    delwin (apad.ptrwin);
 
   /* Store the events and appointments (recursive and normal items). */
-  apad->length = day_store_items (date, &inday->nb_events, &inday->nb_apoints);
+  apad.length = day_store_items (date, &inday->nb_events, &inday->nb_apoints);
 
   /* Create the new pad with its new length. */
   if (day_changed)
-    apad->first_onscreen = 0;
-  apad->ptrwin = newpad (apad->length, apad->width);
+    apad.first_onscreen = 0;
+  apad.ptrwin = newpad (apad.length, apad.width);
 
-  return (inday);
+  return inday;
 }
 
 /*
@@ -329,7 +335,7 @@ display_item_date (int incolor, apoint_llist_node_t *i, int type, long date,
   char a_st[100], a_end[100];
   int recur = 0;
 
-  win = apad->ptrwin;
+  win = apad.ptrwin;
   apoint_sec2str (i, type, date, a_st, a_end);
   if (type == RECUR_EVNT || type == RECUR_APPT)
     recur = 1;
@@ -359,7 +365,7 @@ display_item (int incolor, char *msg, int recur, int note, int len, int y,
   int ch_recur, ch_note;
   char buf[len];
 
-  win = apad->ptrwin;
+  win = apad.ptrwin;
   ch_recur = (recur) ? '*' : ' ';
   ch_note = (note) ? '>' : ' ';
   if (incolor == 0)
@@ -368,7 +374,7 @@ display_item (int incolor, char *msg, int recur, int note, int len, int y,
     mvwprintw (win, y, x, " %c%c%s", ch_recur, ch_note, msg);
   else
     {
-      strncpy (buf, msg, len - 1);
+      (void)strncpy (buf, msg, len - 1);
       buf[len - 1] = '\0';
       mvwprintw (win, y, x, " %c%c%s...", ch_recur, ch_note, buf);
     }
@@ -395,12 +401,6 @@ day_write_pad (long date, int width, int length, int incolor)
   line = item_number = 0;
   max_pos = length;
 
-  /* Initialize the structure used to store highlited item. */
-  if (day_saved_item == NULL)
-    {
-      day_saved_item = malloc (sizeof (struct day_saved_item_s));
-    }
-
   for (p = day_items_ptr; p != 0; p = p->next)
     {
       if (p->type == RECUR_EVNT || p->type == RECUR_APPT)
@@ -413,8 +413,8 @@ day_write_pad (long date, int width, int length, int incolor)
 	  item_number++;
 	  if (item_number - incolor == 0)
 	    {
-	      day_saved_item->type = p->type;
-	      day_saved_item->mesg = p->mesg;
+	      day_saved_item.type = p->type;
+	      day_saved_item.mesg = p->mesg;
 	    }
 	  display_item (item_number - incolor, p->mesg, recur,
 			(p->note != NULL) ? 1 : 0, width - 7, line, x_pos);
@@ -426,8 +426,8 @@ day_write_pad (long date, int width, int length, int incolor)
 	  /* Draw a line between events and appointments. */
 	  if (line > 0 && draw_line)
 	    {
-	      wmove (apad->ptrwin, line, 0);
-	      whline (apad->ptrwin, 0, width);
+	      wmove (apad.ptrwin, line, 0);
+	      whline (apad.ptrwin, 0, width);
 	      draw_line = false;
 	    }
 	  /* Last print the appointments for current day. */
@@ -435,10 +435,10 @@ day_write_pad (long date, int width, int length, int incolor)
 	  day_item_s2apoint_s (&a, p);
 	  if (item_number - incolor == 0)
 	    {
-	      day_saved_item->type = p->type;
-	      day_saved_item->mesg = p->mesg;
+	      day_saved_item.type = p->type;
+	      day_saved_item.mesg = p->mesg;
 	      apoint_sec2str (&a, p->type, date,
-			      day_saved_item->start, day_saved_item->end);
+			      day_saved_item.start, day_saved_item.end);
 	    }
 	  display_item_date (item_number - incolor, &a, p->type,
 			     date, line + 1, x_pos);
@@ -454,11 +454,11 @@ day_write_pad (long date, int width, int length, int incolor)
 void
 day_popup_item (void)
 {
-  if (day_saved_item->type == EVNT || day_saved_item->type == RECUR_EVNT)
-    item_in_popup (NULL, NULL, day_saved_item->mesg, _("Event :"));
-  else if (day_saved_item->type == APPT || day_saved_item->type == RECUR_APPT)
-    item_in_popup (day_saved_item->start, day_saved_item->end,
-		   day_saved_item->mesg, _("Appointment :"));
+  if (day_saved_item.type == EVNT || day_saved_item.type == RECUR_EVNT)
+    item_in_popup (NULL, NULL, day_saved_item.mesg, _("Event :"));
+  else if (day_saved_item.type == APPT || day_saved_item.type == RECUR_APPT)
+    item_in_popup (day_saved_item.start, day_saved_item.end,
+		   day_saved_item.mesg, _("Appointment :"));
   else
     EXIT (_("unknown item type"));
   /* NOTREACHED */
@@ -545,8 +545,8 @@ update_start_time (long *start, long *dur)
   do
     {
       timestr = day_edit_time (*start);
-      sscanf (timestr, "%u:%u", &hr, &mn);
-      free (timestr);
+      (void)sscanf (timestr, "%u:%u", &hr, &mn);
+      mem_free (timestr);
       newtime = update_time_in_date (*start, hr, mn);
       if (newtime < *start + *dur)
 	{
@@ -572,8 +572,8 @@ update_duration (long *start, long *dur)
   char *timestr;
 
   timestr = day_edit_time (*start + *dur);
-  sscanf (timestr, "%u:%u", &hr, &mn);
-  free (timestr);
+  (void)sscanf (timestr, "%u:%u", &hr, &mn);
+  mem_free (timestr);
   newtime = update_time_in_date (*start, hr, mn);
   *dur = (newtime > *start) ? newtime - *start : DAYINSEC + newtime - *start;
 }
@@ -606,18 +606,18 @@ update_rept (struct rpt_s **rpt, const long start, conf_t *conf)
   do
     {
       status_mesg (msg_rpt_type, msg_rpt_ans);
-      typstr = (char *) malloc (sizeof (char) * SINGLECHAR);
-      snprintf (typstr, SINGLECHAR, "%c", recur_def2char ((*rpt)->type));
+      typstr = (char *) mem_calloc (SINGLECHAR, sizeof (char));
+      (void)snprintf (typstr, SINGLECHAR, "%c", recur_def2char ((*rpt)->type));
       cancel = updatestring (win[STA].p, &typstr, 0, 1);
       if (cancel)
 	{
-	  free (typstr);
+	  mem_free (typstr);
 	  return;
 	}
       else
 	{
 	  ch = toupper (*typstr);
-	  free (typstr);
+	  mem_free (typstr);
 	}
     }
   while ((ch != 'D') && (ch != 'W') && (ch != 'M') && (ch != 'Y'));
@@ -625,18 +625,18 @@ update_rept (struct rpt_s **rpt, const long start, conf_t *conf)
   do
     {
       status_mesg (_("Enter the new repetition frequence:"), "");
-      freqstr = (char *) malloc (BUFSIZ);
-      snprintf (freqstr, BUFSIZ, "%d", (*rpt)->freq);
+      freqstr = (char *) mem_malloc (BUFSIZ);
+      (void)snprintf (freqstr, BUFSIZ, "%d", (*rpt)->freq);
       cancel = updatestring (win[STA].p, &freqstr, 0, 1);
       if (cancel)
 	{
-	  free (freqstr);
+	  mem_free (freqstr);
 	  return;
 	}
       else
 	{
 	  newfreq = atoi (freqstr);
-	  free (freqstr);
+	  mem_free (freqstr);
 	  if (newfreq == 0)
 	    {
 	      status_mesg (msg_wrong_freq, msg_enter);
@@ -648,15 +648,15 @@ update_rept (struct rpt_s **rpt, const long start, conf_t *conf)
 
   do
     {
-      snprintf (outstr, BUFSIZ, "Enter the new ending date: [%s] or '0'",
-		DATEFMT_DESC (conf->input_datefmt));
+      (void)snprintf (outstr, BUFSIZ, "Enter the new ending date: [%s] or '0'",
+                      DATEFMT_DESC (conf->input_datefmt));
       status_mesg (_(outstr), "");
       timstr =
 	  date_sec2date_str ((*rpt)->until, DATEFMT (conf->input_datefmt));
       cancel = updatestring (win[STA].p, &timstr, 0, 1);
       if (cancel)
 	{
-	  free (timstr);
+	  mem_free (timstr);
 	  return;
 	}
       if (strcmp (timstr, "0") == 0)
@@ -691,8 +691,8 @@ update_rept (struct rpt_s **rpt, const long start, conf_t *conf)
 	    }
 	  else
 	    {
-	      snprintf (outstr, BUFSIZ, msg_fmts,
-			DATEFMT_DESC (conf->input_datefmt));
+	      (void)snprintf (outstr, BUFSIZ, msg_fmts,
+                              DATEFMT_DESC (conf->input_datefmt));
 	      status_mesg (msg_wrong_date, _(outstr));
 	      (void)wgetch (win[STA].p);
 	      date_entered = 0;
@@ -701,7 +701,7 @@ update_rept (struct rpt_s **rpt, const long start, conf_t *conf)
     }
   while (date_entered == 0);
 
-  free (timstr);
+  mem_free (timstr);
   (*rpt)->type = recur_char2def (ch);
   (*rpt)->freq = newfreq;
   (*rpt)->until = newuntil;
@@ -884,6 +884,70 @@ day_erase_item (long date, int item_number, erase_flag_e flag)
     return (p->type);
 }
 
+/* Cut an item so it can be pasted somewhere else later. */
+int
+day_cut_item (long date, int item_number)
+{
+  const int DELETE_WHOLE = 1;
+  struct day_item_s *p;
+  
+  p = day_get_item (item_number);
+  switch (p->type)
+    {
+    case EVNT:
+      event_delete_bynum (date, day_item_nb (date, item_number, EVNT),
+                          ERASE_CUT);      
+      break;
+    case RECUR_EVNT:
+      recur_event_erase (date, day_item_nb (date, item_number, RECUR_EVNT),
+                         DELETE_WHOLE, ERASE_CUT);
+      break;
+    case APPT:
+      apoint_delete_bynum (date, day_item_nb (date, item_number, APPT),
+                           ERASE_CUT);
+      break;
+    case RECUR_APPT:
+      recur_apoint_erase (date, p->appt_pos, DELETE_WHOLE, ERASE_CUT);
+      break;
+    default:
+      EXIT (_("unknwon type"));
+      /* NOTREACHED */
+    }
+
+  return p->type;
+}
+
+/* Paste a previously cut item. */
+int
+day_paste_item (long date, int cut_item_type)
+{
+  int pasted_item_type;
+
+  pasted_item_type = cut_item_type;
+  switch (cut_item_type)
+    {
+    case 0:
+      return 0;
+    case EVNT:
+      event_paste_item ();
+      break;
+    case RECUR_EVNT:
+      recur_event_paste_item ();
+      break;
+    case APPT:
+      apoint_paste_item ();
+      break;
+    case RECUR_APPT:
+      recur_apoint_paste_item ();
+      break;
+    default:
+      EXIT (_("unknwon type"));
+      /* NOTREACHED */
+    }
+
+  return pasted_item_type;
+}
+
 /* Returns a structure containing the selected item. */
 struct day_item_s *
 day_get_item (int item_number)
@@ -943,7 +1007,7 @@ day_edit_note (char *editor)
       else
 	p->note = filename;
     }
-  snprintf (fullname, BUFSIZ, "%s%s", path_notes, p->note);
+  (void)snprintf (fullname, BUFSIZ, "%s%s", path_notes, p->note);
   wins_launch_external (fullname, editor);
 
   date = calendar_get_slctd_day_sec ();
@@ -978,6 +1042,6 @@ day_view_note (char *pager)
   p = day_get_item (apoint_hilt ());
   if (p->note == NULL)
     return;
-  snprintf (fullname, BUFSIZ, "%s%s", path_notes, p->note);
+  (void)snprintf (fullname, BUFSIZ, "%s%s", path_notes, p->note);
   wins_launch_external (fullname, pager);
 }
