@@ -1,8 +1,8 @@
-/*	$calcurse: day.h,v 1.19 2008/04/12 21:14:03 culot Exp $	*/
+/*	$calcurse: day.h,v 1.22 2009/01/03 21:32:11 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2008 Frederic Culot
+ * Copyright (c) 2004-2009 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,12 +67,15 @@ struct day_saved_item_s
   char *mesg;
 };
 
+void               day_free_list (void);
 day_items_nb_t    *day_process_storage (date_t *, bool, day_items_nb_t *);
 void               day_write_pad (long, int, int, int);
 void               day_popup_item (void);
 int                day_check_if_item (date_t);
 void               day_edit_item (conf_t *);
 int                day_erase_item (long, int, erase_flag_e);
+int                day_cut_item (long, int);
+int                day_paste_item (long, int);
 struct day_item_s *day_get_item (int);
 int                day_item_nb (long, int, int);
 void               day_edit_note (char *);
