@@ -1,8 +1,8 @@
-/*	$calcurse: event.c,v 1.8 2008/12/14 15:54:51 culot Exp $	*/
+/*	$calcurse: event.c,v 1.12 2009/01/03 21:32:11 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2008 Frederic Culot
+ * Copyright (c) 2004-2009 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,21 +31,71 @@
 
 #include "vars.h"
 #include "i18n.h"
+#include "mem.h"
 #include "event.h"
 
-struct event_s *eventlist;
+struct event_s        *eventlist;
+static struct event_s  bkp_cut_event;
+
+void
+event_free_bkp (erase_flag_e flag)
+{
+  if (bkp_cut_event.mesg)
+    {
+      mem_free (bkp_cut_event.mesg);
+      bkp_cut_event.mesg = 0;
+    }
+  if (bkp_cut_event.note)
+    {
+      if (flag == ERASE_FORCE)
+        erase_note (&bkp_cut_event.note, ERASE_FORCE);
+      else
+        {
+          mem_free (bkp_cut_event.note);
+          bkp_cut_event.note = 0;
+        }
+    }
+}
+
+static void
+event_dup (struct event_s *in, struct event_s *bkp)
+{
+  EXIT_IF (!in || !bkp, _("null pointer"));
+
+  bkp->id = in->id;
+  bkp->day = in->day;
+  bkp->mesg = mem_strdup (in->mesg);
+  if (in->note)
+    bkp->note = mem_strdup (in->note);
+}
+
+void
+event_llist_free (void)
+{
+  struct event_s *o, **i;
+
+  i = &eventlist;
+  while (*i)
+    {
+      o = *i;
+      *i = o->next;
+      mem_free (o->mesg);
+      if (o->note)
+        mem_free (o->note);
+      mem_free (o);
+    }
+}
 
 /* Create a new event */
 struct event_s *
 event_new (char *mesg, char *note, long day, int id)
 {
   struct event_s *o, **i;
-  o = (struct event_s *) malloc (sizeof (struct event_s));
-  o->mesg = (char *) malloc (strlen (mesg) + 1);
-  strncpy (o->mesg, mesg, strlen (mesg) + 1);
+  o = (struct event_s *) mem_malloc (sizeof (struct event_s));
+  o->mesg = mem_strdup (mesg);
   o->day = day;
   o->id = id;
-  o->note = (note != NULL) ? strdup (note) : NULL;
+  o->note = (note != NULL) ? mem_strdup (note) : NULL;
   i = &eventlist;
   for (;;)
     {
@@ -80,26 +130,25 @@ event_write (struct event_s *o, FILE *f)
 
   t = o->day;
   lt = localtime (&t);
-  fprintf (f, "%02u/%02u/%04u [%d] ", lt->tm_mon + 1, lt->tm_mday,
-           1900 + lt->tm_year, o->id);
+  (void)fprintf (f, "%02u/%02u/%04u [%d] ", lt->tm_mon + 1, lt->tm_mday,
+                 1900 + lt->tm_year, o->id);
   if (o->note != NULL)
-    fprintf (f, ">%s ", o->note);
-  fprintf (f, "%s\n", o->mesg);
+    (void)fprintf (f, ">%s ", o->note);
+  (void)fprintf (f, "%s\n", o->mesg);
 }
 
 /* Load the events from file */
 struct event_s *
 event_scan (FILE *f, struct tm start, int id, char *note)
 {
-  struct tm *lt;
   char buf[MESG_MAXSIZE], *nl;
   time_t tstart, t;
 
   t = time (NULL);
-  lt = localtime (&t);
+  (void)localtime (&t);
 
   /* Read the event description */
-  fgets (buf, MESG_MAXSIZE, f);
+  (void)fgets (buf, MESG_MAXSIZE, f);
   nl = strchr (buf, '\n');
   if (nl)
     {
@@ -155,15 +204,25 @@ event_delete_bynum (long start, unsigned num, erase_flag_e flag)
 	{
 	  if (n == num)
 	    {
-	      if (flag == ERASE_FORCE_ONLY_NOTE)
-		erase_note (&i->note, flag);
-	      else
-		{
+              switch (flag)
+                {
+                case ERASE_FORCE_ONLY_NOTE:
+                  erase_note (&i->note, flag);
+                  break;
+                case ERASE_CUT:
+                  event_free_bkp (ERASE_FORCE);
+                  event_dup (i, &bkp_cut_event);
+                  if (i->note)
+                    mem_free (i->note);
+                  /* FALLTHROUGH */
+                default:
 		  *iptr = i->next;
-		  free (i->mesg);
-		  erase_note (&i->note, flag);
-		  free (i);
-		}
+		  mem_free (i->mesg);
+                  if (flag != ERASE_FORCE_KEEP_NOTE && flag != ERASE_CUT)
+                    erase_note (&i->note, flag);
+		  mem_free (i);
+                  break;
+                }
 	      return;
 	    }
 	  n++;
@@ -173,3 +232,12 @@ event_delete_bynum (long start, unsigned num, erase_flag_e flag)
   EXIT (_("event not found"));
   /* NOTREACHED */  
 }
+
+void
+event_paste_item (void)
+{
+  (void)event_new (bkp_cut_event.mesg, bkp_cut_event.note,
+                   date2sec (*calendar_get_slctd_day (), 12, 0),
+                   bkp_cut_event.id);
+  event_free_bkp (ERASE_FORCE_KEEP_NOTE);
+}
