@@ -1,8 +1,8 @@
-/*	$calcurse: help.h,v 1.5 2008/04/12 21:14:03 culot Exp $	*/
+/*	$calcurse: help.h,v 1.9 2008/12/20 19:27:31 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2007 Frederic Culot
+ * Copyright (c) 2004-2008 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,13 +27,16 @@
 #ifndef CALCURSE_HELP_H
 #define CALCURSE_HELP_H
 
-typedef struct
-{
-  char *title;
-  char *text;
-}
-help_page_t;
+#include "wins.h"
 
+#define HELPTEXTSIZ 4096
+
+typedef struct {
+  char *title;
+  char text[HELPTEXTSIZ];
+} help_page_t;
+
+void help_wins_init (scrollwin_t *, int, int, int, int);
 void help_screen (void);
 
 #endif /* CALCURSE_HELP_H */
