@@ -1,8 +1,8 @@
-/*	$calcurse: keys.c,v 1.10 2008/12/15 20:02:00 culot Exp $	*/
+/*	$calcurse: keys.c,v 1.16 2009/01/24 18:45:35 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2008 Frederic Culot
+ * Copyright (c) 2008-2009 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include "i18n.h"
 #include "utils.h"
 #include "custom.h"
+#include "mem.h"
 #include "keys.h"
 
 #define MAXKEYVAL   KEY_MAX  /* ncurses defines KEY_MAX as maximum key value */
@@ -55,6 +56,8 @@ static struct keydef_s keydef[NBKEYS] = {
   {"generic-help", "?"},
   {"generic-quit", "q Q"},
   {"generic-save", "s S C-s"},
+  {"generic-cut", "C-x"},
+  {"generic-paste", "C-v"},
   {"generic-change-view", "TAB"},
   {"generic-import", "i I"},
   {"generic-export", "x X"},
@@ -108,12 +111,14 @@ dump_intro (FILE *fd)
       "'C-'.\n"
       "# The escape, space bar and horizontal Tab key can be specified using\n"
       "# the 'ESC', 'SPC' and 'TAB' keyword, respectively.\n"
-      "# Arrow keys can also be specified with the UP, DWN, LFT, RGT keywords."
+      "# Arrow keys can also be specified with the UP, DWN, LFT, RGT keywords.\n"
+      "# Last, Home and End keys can be assigned using 'KEY_HOME' and 'KEY_END'\n"
+      "# keywords."
       "\n#\n"
       "# A description of what each ACTION keyword is used for is available\n"
       "# from calcurse online configuration menu.\n");
 
-  fprintf (fd, "%s\n", intro);
+  (void)fprintf (fd, "%s\n", intro);
 }
 
 void
@@ -126,6 +131,28 @@ keys_init (void)
   bzero (keys, NBKEYS);
 }
 
+void
+keys_free (void)
+{
+  struct key_str_s *o, **i;
+  int key;
+
+  for (key = 0; key < NBKEYS; key++)
+    {
+      if (keys[key] == 0)
+        continue;
+
+      i = &keys[key];
+      while (*i)
+        {
+          o = *i;
+          *i = o->next;
+          mem_free (o->str);
+          mem_free (o);
+        }
+    }
+}
+
 void
 keys_dump_defaults (char *file)
 {
@@ -137,8 +164,8 @@ keys_dump_defaults (char *file)
 
   dump_intro (fd);
   for (i = 0; i < NBKEYS; i++)
-    fprintf (fd, "%s  %s\n", keydef[i].label, keydef[i].binding);
-  fclose (fd);
+    (void)fprintf (fd, "%s  %s\n", keydef[i].label, keydef[i].binding);
+  file_close (fd, __FILE_POS__);
 }
 
 char *
@@ -182,8 +209,8 @@ add_key_str (keys_e action, int key)
   if (action < 0 || action > NBKEYS)
     return;
 
-  new = malloc (sizeof (struct key_str_s));
-  new->str = strdup (keys_int2str (key));
+  new = mem_malloc (sizeof (struct key_str_s));
+  new->str = mem_strdup (keys_int2str (key));
   new->next = NULL;
   i = &keys[action];
   for (;;)
@@ -221,20 +248,17 @@ del_key_str (keys_e action, int key)
 {
   struct key_str_s *old, **i;
   char oldstr[BUFSIZ];
-  int oldstrlen;
   
   if (action < 0 || action > NBKEYS)
     return;
 
-  strncpy (oldstr, keys_int2str (key), BUFSIZ);
-  oldstrlen = strlen (oldstr);
+  (void)strncpy (oldstr, keys_int2str (key), BUFSIZ);
   for (i = &keys[action]; *i; i = &(*i)->next)
     {
-      if (strlen ((*i)->str) == oldstrlen
-          && !(strncmp ((*i)->str, oldstr, oldstrlen)))
+      if (!strcmp ((*i)->str, oldstr))
         {
-          old = (*i);
-          (*i) = old->next;
+          old = *i;
+          *i = old->next;
           mem_free (old->str);          
           mem_free (old);
           break;
@@ -265,6 +289,8 @@ keys_str2int (char *key)
   const string_t CURSES_KEY_DOWN = STRING_BUILD ("DWN");
   const string_t CURSES_KEY_LEFT = STRING_BUILD ("LFT");
   const string_t CURSES_KEY_RIGHT = STRING_BUILD ("RGT");
+  const string_t CURSES_KEY_HOME = STRING_BUILD ("KEY_HOME");
+  const string_t CURSES_KEY_END = STRING_BUILD ("KEY_END");
 
   if (!key)
     return -1;
@@ -290,6 +316,10 @@ keys_str2int (char *key)
         return KEY_LEFT;
       else if (!strncmp (key, CURSES_KEY_RIGHT.str, CURSES_KEY_RIGHT.len))
         return KEY_RIGHT;
+      else if (!strncmp (key, CURSES_KEY_HOME.str, CURSES_KEY_HOME.len))
+        return KEY_HOME;
+      else if (!strncmp (key, CURSES_KEY_END.str, CURSES_KEY_END.len))
+        return KEY_END;
       else
         return -1;
     }
@@ -314,6 +344,10 @@ keys_int2str (int key)
       return "LFT";
     case KEY_RIGHT:
       return "RGT";
+    case KEY_HOME:
+      return "KEY_HOME";
+    case KEY_END:
+      return "KEY_END";
     default:
       return (char *)keyname (key);
     }
@@ -367,8 +401,8 @@ keys_action_allkeys (keys_e action)
   for (i = keys[action]; i; i = i->next)
     {
       const int MAXLEN = sizeof (keystr) - 1 - strlen (keystr);
-      strncat (keystr, i->str, MAXLEN - 1);
-      strncat (keystr, CHAR_SPACE, 1);
+      (void)strncat (keystr, i->str, MAXLEN - 1);
+      (void)strncat (keystr, CHAR_SPACE, 1);
     }
 
   return keystr;
@@ -388,18 +422,18 @@ keys_format_label (char *key, int keylen)
 
   bzero (fmtkey, sizeof (fmtkey));
   if (len == 0)
-    snprintf (fmtkey, sizeof (fmtkey), "?");
+    (void)snprintf (fmtkey, sizeof (fmtkey), "?");
   else if (len <= keylen)
     {
       for (i = 0; i < keylen - len; i++)
         fmtkey[i] = ' ';
-      strncat (fmtkey, key, keylen);
+      (void)strncat (fmtkey, key, keylen);
     }
   else
     {
       for (i = 0; i < keylen - 1; i++)
         fmtkey[i] = key[i];
-      strncat (fmtkey, dot, strlen (dot));
+      (void)strncat (fmtkey, dot, strlen (dot));
     }
   return fmtkey;
 }
@@ -416,21 +450,22 @@ keys_display_bindings_bar (WINDOW *win, binding_t **binding, int first_key,
   cmdlen += space_between_cmds;
   
   j = 0;
-  erase_status_bar ();
+  wins_erase_status_bar ();
   for (i = first_key; i < last_key; i += 2)
     {
       char key[KEYS_KEYLEN + 1], *fmtkey;
       const int KEY_POS = j * cmdlen;
       const int LABEL_POS = j * cmdlen + KEYS_KEYLEN + 1;
 
-      strncpy (key, keys_action_firstkey (binding[i]->action), KEYS_KEYLEN);
+      (void)strncpy (key, keys_action_firstkey (binding[i]->action),
+                     KEYS_KEYLEN);
       fmtkey = keys_format_label (key, KEYS_KEYLEN);
       custom_apply_attr (win, ATTR_HIGHEST);
       mvwprintw (win, 0, KEY_POS, fmtkey);
       if (i + 1 != last_key)
         {
-          strncpy (key, keys_action_firstkey (binding[i + 1]->action),
-                   KEYS_KEYLEN);
+          (void)strncpy (key, keys_action_firstkey (binding[i + 1]->action),
+                         KEYS_KEYLEN);
           fmtkey = keys_format_label (key, KEYS_KEYLEN);
           mvwprintw (win, 1, KEY_POS, fmtkey);
         }
@@ -541,7 +576,7 @@ keys_popup_info (keys_e key)
 #define WINCOL (col - 4)
   infowin = popup (WINROW, WINCOL, (row - WINROW) / 2, (col - WINCOL) / 2,
                    keydef[key].label, info[key], 1);
-  keys_getch (infowin);
+  (void)keys_getch (infowin);
   delwin (infowin);
 #undef WINROW
 #undef WINCOL
@@ -555,7 +590,7 @@ keys_save_bindings (FILE *fd)
   EXIT_IF (fd == NULL, _("FATAL ERROR: null file pointer."));
   dump_intro (fd);
   for (i = 0; i < NBKEYS; i++)
-    fprintf (fd, "%s  %s\n", keydef[i].label, keys_action_allkeys (i));
+    (void)fprintf (fd, "%s  %s\n", keydef[i].label, keys_action_allkeys (i));
 }
 
 int
@@ -570,3 +605,41 @@ keys_check_missing_bindings (void)
     }
   return 0;
 }
+
+void
+keys_fill_missing (void)
+{
+  int i;
+
+  for (i = 0; i < NBKEYS; i++)
+    {
+      if (keys[i] == 0)
+        {
+          char *p, tmpbuf[BUFSIZ];
+
+          (void)strncpy (tmpbuf, keydef[i].binding, BUFSIZ);          
+          p = tmpbuf;
+          for (;;)
+            {
+              char key_ch[BUFSIZ];
+
+              while (*p == ' ')
+                p++;
+              if (sscanf (p, "%s", key_ch) == 1)
+                {
+                  int ch, used;
+                  
+                  ch = keys_str2int (key_ch);
+                  used = keys_assign_binding (ch, i);
+                  if (used)
+                    WARN_MSG (_("When adding default key for \"%s\", "
+                                "\"%s\" was already assigned!"),
+                              keydef[i].label, key_ch);
+                  p += strlen (key_ch) + 1;
+                }
+              else
+                break;
+            }
+        }
+    }
+}
