@@ -1,8 +1,8 @@
-/*	$calcurse: notify.c,v 1.32 2008/12/18 20:38:52 culot Exp $	*/
+/*	$calcurse: notify.c,v 1.36 2009/01/22 18:11:57 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2008 Frederic Culot
+ * Copyright (c) 2004-2009 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,16 +29,21 @@
 #include <string.h>
 #include <unistd.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include "i18n.h"
 #include "utils.h"
 #include "custom.h"
 #include "keys.h"
+#include "mem.h"
 #include "notify.h"
 
-static struct notify_vars_s *notify = NULL;
-static struct notify_app_s *notify_app = NULL;
-static pthread_attr_t detached_thread_attr;
-static pthread_t notify_t_main;
+static struct notify_vars_s   notify;
+static struct notify_app_s    notify_app;
+static pthread_attr_t         detached_thread_attr;
+static pthread_t              notify_t_main;
 
 /* Return 1 if we need to display the notify-bar, else 0. */
 int
@@ -46,9 +51,9 @@ notify_bar (void)
 {
   int display_bar = 0;
 
-  pthread_mutex_lock (&nbar->mutex);
-  display_bar = (nbar->show) ? 1 : 0;
-  pthread_mutex_unlock (&nbar->mutex);
+  pthread_mutex_lock (&nbar.mutex);
+  display_bar = (nbar.show) ? 1 : 0;
+  pthread_mutex_unlock (&nbar.mutex);
 
   return (display_bar);
 }
@@ -61,16 +66,15 @@ notify_init_vars (void)
   char *date_format = "%a %F";
   char *cmd = "printf '\\a'";
 
-  nbar = (struct nbar_s *) malloc (sizeof (struct nbar_s));
-  pthread_mutex_init (&nbar->mutex, NULL);
-  nbar->show = 1;
-  nbar->cntdwn = 300;
-  strncpy (nbar->datefmt, date_format, strlen (date_format) + 1);
-  strncpy (nbar->timefmt, time_format, strlen (time_format) + 1);
-  strncpy (nbar->cmd, cmd, strlen (cmd) + 1);
+  pthread_mutex_init (&nbar.mutex, NULL);
+  nbar.show = 1;
+  nbar.cntdwn = 300;
+  (void)strncpy (nbar.datefmt, date_format, strlen (date_format) + 1);
+  (void)strncpy (nbar.timefmt, time_format, strlen (time_format) + 1);
+  (void)strncpy (nbar.cmd, cmd, strlen (cmd) + 1);
 
-  if ((nbar->shell = getenv ("SHELL")) == NULL)
-    nbar->shell = "/bin/sh";
+  if ((nbar.shell = getenv ("SHELL")) == NULL)
+    nbar.shell = "/bin/sh";
 
   (void)pthread_attr_init (&detached_thread_attr);
   (void)pthread_attr_setdetachstate (&detached_thread_attr,
@@ -85,11 +89,11 @@ extract_aptsfile (void)
 
   file = strrchr (path_apts, '/');
   if (!file)
-    notify->apts_file = path_apts;
+    notify.apts_file = path_apts;
   else
     {
-      notify->apts_file = file;
-      notify->apts_file++;
+      notify.apts_file = file;
+      notify.apts_file++;
     }
 }
 
@@ -101,12 +105,10 @@ extract_aptsfile (void)
 void
 notify_init_bar (void)
 {
-  notify = (struct notify_vars_s *) malloc (sizeof (struct notify_vars_s));
-  notify_app = (struct notify_app_s *) malloc (sizeof (struct notify_app_s));
-  pthread_mutex_init (&notify->mutex, NULL);
-  pthread_mutex_init (&notify_app->mutex, NULL);
-  notify_app->got_app = 0;
-  notify->win = newwin (win[NOT].h, win[NOT].w, win[NOT].y, win[NOT].x);
+  pthread_mutex_init (&notify.mutex, NULL);
+  pthread_mutex_init (&notify_app.mutex, NULL);
+  notify_app.got_app = 0;
+  notify.win = newwin (win[NOT].h, win[NOT].w, win[NOT].y, win[NOT].x);
   extract_aptsfile ();
 }
 
@@ -114,8 +116,8 @@ notify_init_bar (void)
 void
 notify_stop_main_thread (void)
 {
-  pthread_cancel (notify_t_main);
-  return;
+  if (notify_t_main)
+    pthread_cancel (notify_t_main);
 }
 
 /* 
@@ -125,8 +127,8 @@ notify_stop_main_thread (void)
 void
 notify_reinit_bar (void)
 {
-  delwin (notify->win);
-  notify->win = newwin (win[NOT].h, win[NOT].w, win[NOT].y, win[NOT].x);
+  delwin (notify.win);
+  notify.win = newwin (win[NOT].h, win[NOT].w, win[NOT].y, win[NOT].x);
 }
 
 /* Launch user defined command as a notification. */
@@ -158,77 +160,78 @@ notify_update_bar (void)
   char buf[BUFSIZ];
 
   date_pos = space;
-  pthread_mutex_lock (&notify->mutex);
+  pthread_mutex_lock (&notify.mutex);
 
-  file_pos = strlen (notify->date) + strlen (notify->time) + 7 + 2 * space;
-  app_pos = file_pos + strlen (notify->apts_file) + 2 + space;
+  file_pos = strlen (notify.date) + strlen (notify.time) + 7 + 2 * space;
+  app_pos = file_pos + strlen (notify.apts_file) + 2 + space;
   txt_max_len = col - (app_pos + 12 + space);
 
-  custom_apply_attr (notify->win, ATTR_HIGHEST);
-  wattron (notify->win, A_UNDERLINE | A_REVERSE);
-  mvwhline (notify->win, 0, 0, ACS_HLINE, col);
-  mvwprintw (notify->win, 0, date_pos, "[ %s | %s ]",
-	     notify->date, notify->time);
-  mvwprintw (notify->win, 0, file_pos, "(%s)", notify->apts_file);
+  custom_apply_attr (notify.win, ATTR_HIGHEST);
+  wattron (notify.win, A_UNDERLINE | A_REVERSE);
+  mvwhline (notify.win, 0, 0, ACS_HLINE, col);
+  mvwprintw (notify.win, 0, date_pos, "[ %s | %s ]",
+	     notify.date, notify.time);
+  mvwprintw (notify.win, 0, file_pos, "(%s)", notify.apts_file);
 
-  pthread_mutex_lock (&notify_app->mutex);
-  if (notify_app->got_app)
+  pthread_mutex_lock (&notify_app.mutex);
+  if (notify_app.got_app)
     {
-      if (strlen (notify_app->txt) > txt_max_len)
+      if (strlen (notify_app.txt) > txt_max_len)
 	{
 	  too_long = 1;
-	  strncpy (buf, notify_app->txt, txt_max_len - 3);
+	  (void)strncpy (buf, notify_app.txt, txt_max_len - 3);
 	  buf[txt_max_len - 3] = '\0';
 	}
-      time_left = notify_app->time - notify->time_in_sec;
+      time_left = notify_app.time - notify.time_in_sec;
       if (time_left > 0)
 	{
 	  hours_left = (time_left / HOURINSEC);
 	  minutes_left = (time_left - hours_left * HOURINSEC) / MININSEC;
-	  pthread_mutex_lock (&nbar->mutex);
+	  pthread_mutex_lock (&nbar.mutex);
 
-	  if (time_left < nbar->cntdwn && (notify_app->state & APOINT_NOTIFY))
+	  if (time_left < nbar.cntdwn && (notify_app.state & APOINT_NOTIFY))
 	    blinking = 1;
 	  else
 	    blinking = 0;
 
 	  if (blinking)
-	    wattron (notify->win, A_BLINK);
+	    wattron (notify.win, A_BLINK);
 	  if (too_long)
-	    mvwprintw (notify->win, 0, app_pos, "> %02d:%02d :: %s.. <",
+	    mvwprintw (notify.win, 0, app_pos, "> %02d:%02d :: %s.. <",
 		       hours_left, minutes_left, buf);
 	  else
-	    mvwprintw (notify->win, 0, app_pos, "> %02d:%02d :: %s <",
-		       hours_left, minutes_left, notify_app->txt);
+	    mvwprintw (notify.win, 0, app_pos, "> %02d:%02d :: %s <",
+		       hours_left, minutes_left, notify_app.txt);
 	  if (blinking)
-	    wattroff (notify->win, A_BLINK);
+	    wattroff (notify.win, A_BLINK);
 
-	  if (blinking && !(notify_app->state & APOINT_NOTIFIED))
+	  if (blinking && !(notify_app.state & APOINT_NOTIFIED))
 	    {
-	      notify_app->state |= APOINT_NOTIFIED;
-	      launch_cmd (nbar->cmd, nbar->shell);
+	      notify_app.state |= APOINT_NOTIFIED;
+	      launch_cmd (nbar.cmd, nbar.shell);
 	    }
-	  pthread_mutex_unlock (&nbar->mutex);
+	  pthread_mutex_unlock (&nbar.mutex);
 	}
       else
 	{
-	  notify_app->got_app = 0;
-	  pthread_mutex_unlock (&notify_app->mutex);
-	  pthread_mutex_unlock (&notify->mutex);
+	  notify_app.got_app = 0;
+	  pthread_mutex_unlock (&notify_app.mutex);
+	  pthread_mutex_unlock (&notify.mutex);
 	  notify_check_next_app ();
 	  return;
 	}
     }
-  pthread_mutex_unlock (&notify_app->mutex);
+  pthread_mutex_unlock (&notify_app.mutex);
 
-  wattroff (notify->win, A_UNDERLINE | A_REVERSE);
-  custom_remove_attr (notify->win, ATTR_HIGHEST);
-  wrefresh (notify->win);
+  wattroff (notify.win, A_UNDERLINE | A_REVERSE);
+  custom_remove_attr (notify.win, ATTR_HIGHEST);
+  wrefresh (notify.win);
 
-  pthread_mutex_unlock (&notify->mutex);
+  pthread_mutex_unlock (&notify.mutex);
 }
 
 /* Update the notication bar content */
+/* ARGSUSED0 */
 static void *
 notify_main_thread (void *arg)
 {
@@ -245,22 +248,22 @@ notify_main_thread (void *arg)
     {
       ntimer = time (NULL);
       ntime = localtime (&ntimer);
-      pthread_mutex_lock (&notify->mutex);
-      notify->time_in_sec = ntimer;
-      pthread_mutex_lock (&nbar->mutex);
-      strftime (notify->time, NOTIFY_FIELD_LENGTH, nbar->timefmt, ntime);
-      strftime (notify->date, NOTIFY_FIELD_LENGTH, nbar->datefmt, ntime);
-      pthread_mutex_unlock (&nbar->mutex);
-      pthread_mutex_unlock (&notify->mutex);
+      pthread_mutex_lock (&notify.mutex);
+      notify.time_in_sec = ntimer;
+      pthread_mutex_lock (&nbar.mutex);
+      strftime (notify.time, NOTIFY_FIELD_LENGTH, nbar.timefmt, ntime);
+      strftime (notify.date, NOTIFY_FIELD_LENGTH, nbar.datefmt, ntime);
+      pthread_mutex_unlock (&nbar.mutex);
+      pthread_mutex_unlock (&notify.mutex);
       notify_update_bar ();
-      sleep (thread_sleep);
+      (void)sleep (thread_sleep);
       elapse += thread_sleep;
       if (elapse >= check_app)
 	{
 	  elapse = 0;
-	  pthread_mutex_lock (&notify_app->mutex);
-	  got_app = notify_app->got_app;
-	  pthread_mutex_unlock (&notify_app->mutex);
+	  pthread_mutex_lock (&notify_app.mutex);
+	  got_app = notify_app.got_app;
+	  pthread_mutex_unlock (&notify_app.mutex);
 	  if (!got_app)
 	    notify_check_next_app ();
 	}
@@ -269,6 +272,7 @@ notify_main_thread (void *arg)
 }
 
 /* Look for the next appointment within the next 24 hours. */
+/* ARGSUSED0 */
 static void *
 notify_thread_app (void *arg)
 {
@@ -285,22 +289,22 @@ notify_thread_app (void *arg)
   tmp_app = *recur_apoint_check_next (&tmp_app, current_time, get_today ());
   tmp_app = *apoint_check_next (&tmp_app, current_time);
 
-  pthread_mutex_lock (&notify_app->mutex);
+  pthread_mutex_lock (&notify_app.mutex);
   if (tmp_app.got_app)
     {
-      notify_app->got_app = 1;
-      notify_app->time = tmp_app.time;
-      notify_app->txt = strdup (tmp_app.txt);
-      notify_app->state = tmp_app.state;
+      notify_app.got_app = 1;
+      notify_app.time = tmp_app.time;
+      notify_app.txt = mem_strdup (tmp_app.txt);
+      notify_app.state = tmp_app.state;
     }
   else
     {
-      notify_app->got_app = 0;
+      notify_app.got_app = 0;
     }
-  pthread_mutex_unlock (&notify_app->mutex);
+  pthread_mutex_unlock (&notify_app.mutex);
 
   if (tmp_app.txt != NULL)
-    free (tmp_app.txt);
+    mem_free (tmp_app.txt);
   notify_update_bar ();
 
   pthread_exit ((void *) 0);
@@ -326,28 +330,28 @@ notify_check_added (char *mesg, long start, char state)
   long gap;
 
   current_time = time (NULL);
-  pthread_mutex_lock (&notify_app->mutex);
-  if (!notify_app->got_app)
+  pthread_mutex_lock (&notify_app.mutex);
+  if (!notify_app.got_app)
     {
       gap = start - current_time;
       if (gap >= 0 && gap <= DAYINSEC)
 	update_notify = 1;
     }
-  else if (start < notify_app->time && start >= current_time)
+  else if (start < notify_app.time && start >= current_time)
     {
       update_notify = 1;
     }
-  else if (start == notify_app->time && state != notify_app->state)
+  else if (start == notify_app.time && state != notify_app.state)
     update_notify = 1;
 
   if (update_notify)
     {
-      notify_app->got_app = 1;
-      notify_app->time = start;
-      notify_app->txt = strdup (mesg);
-      notify_app->state = state;
+      notify_app.got_app = 1;
+      notify_app.time = start;
+      notify_app.txt = mem_strdup (mesg);
+      notify_app.state = state;
     }
-  pthread_mutex_unlock (&notify_app->mutex);
+  pthread_mutex_unlock (&notify_app.mutex);
   notify_update_bar ();
 }
 
@@ -360,33 +364,33 @@ notify_check_repeated (recur_apoint_llist_node_t *i)
   time_t current_time;
 
   current_time = time (NULL);
-  pthread_mutex_lock (&notify_app->mutex);
+  pthread_mutex_lock (&notify_app.mutex);
   if ((real_app_time = recur_item_inday (i->start, i->exc, i->rpt->type,
 					 i->rpt->freq, i->rpt->until,
 					 get_today ()) > current_time))
     {
-      if (!notify_app->got_app)
+      if (!notify_app.got_app)
 	{
 	  if (real_app_time - current_time <= DAYINSEC)
 	    update_notify = 1;
 	}
-      else if (real_app_time < notify_app->time &&
+      else if (real_app_time < notify_app.time &&
 	       real_app_time >= current_time)
 	{
 	  update_notify = 1;
 	}
-      else if (real_app_time == notify_app->time &&
-	       i->state != notify_app->state)
+      else if (real_app_time == notify_app.time &&
+	       i->state != notify_app.state)
 	update_notify = 1;
     }
   if (update_notify)
     {
-      notify_app->got_app = 1;
-      notify_app->time = real_app_time;
-      notify_app->txt = strdup (i->mesg);
-      notify_app->state = i->state;
+      notify_app.got_app = 1;
+      notify_app.time = real_app_time;
+      notify_app.txt = mem_strdup (i->mesg);
+      notify_app.state = i->state;
     }
-  pthread_mutex_unlock (&notify_app->mutex);
+  pthread_mutex_unlock (&notify_app.mutex);
   notify_update_bar ();
 }
 
@@ -395,12 +399,12 @@ notify_same_item (long time)
 {
   int same = 0;
 
-  pthread_mutex_lock (&(notify_app->mutex));
-  if (notify_app->got_app && notify_app->time == time)
+  pthread_mutex_lock (&(notify_app.mutex));
+  if (notify_app.got_app && notify_app.time == time)
     same = 1;
-  pthread_mutex_unlock (&(notify_app->mutex));
+  pthread_mutex_unlock (&(notify_app.mutex));
 
-  return (same);
+  return same;
 }
 
 int
@@ -411,12 +415,12 @@ notify_same_recur_item (recur_apoint_llist_node_t *i)
 
   item_start = recur_item_inday (i->start, i->exc, i->rpt->type,
 				 i->rpt->freq, i->rpt->until, get_today ());
-  pthread_mutex_lock (&notify_app->mutex);
-  if (notify_app->got_app && item_start == notify_app->time)
+  pthread_mutex_lock (&notify_app.mutex);
+  if (notify_app.got_app && item_start == notify_app.time)
     same = 1;
-  pthread_mutex_unlock (&(notify_app->mutex));
+  pthread_mutex_unlock (&(notify_app.mutex));
 
-  return (same);
+  return same;
 }
 
 /* Launch the notify-bar main thread. */
@@ -425,7 +429,6 @@ notify_start_main_thread (void)
 {
   pthread_create (&notify_t_main, NULL, notify_main_thread, NULL);
   notify_check_next_app ();
-  return;
 }
 
 /* Print options related to the notify-bar. */
@@ -452,39 +455,39 @@ notify_print_options (WINDOW *optwin, int col)
   y_offset = 3;
   maxcol = col - 2;
 
-  strncpy (opt[SHOW].name, _("notify-bar_show = "), BUFSIZ);
-  strncpy (opt[DATE].name, _("notify-bar_date = "), BUFSIZ);
-  strncpy (opt[CLOCK].name, _("notify-bar_clock = "), BUFSIZ);
-  strncpy (opt[WARN].name, _("notify-bar_warning = "), BUFSIZ);
-  strncpy (opt[CMD].name, _("notify-bar_command = "), BUFSIZ);
+  (void)strncpy (opt[SHOW].name, _("notify-bar_show = "), BUFSIZ);
+  (void)strncpy (opt[DATE].name, _("notify-bar_date = "), BUFSIZ);
+  (void)strncpy (opt[CLOCK].name, _("notify-bar_clock = "), BUFSIZ);
+  (void)strncpy (opt[WARN].name, _("notify-bar_warning = "), BUFSIZ);
+  (void)strncpy (opt[CMD].name, _("notify-bar_command = "), BUFSIZ);
 
-  strncpy (opt[SHOW].desc,
-	   _("(if set to YES, notify-bar will be displayed)"), BUFSIZ);
-  strncpy (opt[DATE].desc,
-	   _("(Format of the date to be displayed inside notify-bar)"),
-	   BUFSIZ);
-  strncpy (opt[CLOCK].desc,
-	   _("(Format of the time to be displayed inside notify-bar)"),
-	   BUFSIZ);
-  strncpy (opt[WARN].desc,
-	   _("(Warn user if an appointment is within next 'notify-bar_warning'"
-             " seconds)"), BUFSIZ);
-  strncpy (opt[CMD].desc,
-	   _("(Command used to notify user of an upcoming appointment)"),
-	   BUFSIZ);
+  (void)strncpy (opt[SHOW].desc,
+                 _("(if set to YES, notify-bar will be displayed)"), BUFSIZ);
+  (void)strncpy (opt[DATE].desc,
+                 _("(Format of the date to be displayed inside notify-bar)"),
+                 BUFSIZ);
+  (void)strncpy (opt[CLOCK].desc,
+                 _("(Format of the time to be displayed inside notify-bar)"),
+                 BUFSIZ);
+  (void)strncpy (opt[WARN].desc,
+                 _("(Warn user if an appointment is within next "
+                   "'notify-bar_warning' seconds)"), BUFSIZ);
+  (void)strncpy (opt[CMD].desc,
+                 _("(Command used to notify user of an upcoming appointment)"),
+                 BUFSIZ);
 
-  pthread_mutex_lock (&nbar->mutex);
+  pthread_mutex_lock (&nbar.mutex);
 
-  strncpy (opt[DATE].value, nbar->datefmt, BUFSIZ);
-  strncpy (opt[CLOCK].value, nbar->timefmt, BUFSIZ);
-  snprintf (opt[WARN].value, BUFSIZ, "%d", nbar->cntdwn);
-  strncpy (opt[CMD].value, nbar->cmd, BUFSIZ);
+  (void)strncpy (opt[DATE].value, nbar.datefmt, BUFSIZ);
+  (void)strncpy (opt[CLOCK].value, nbar.timefmt, BUFSIZ);
+  (void)snprintf (opt[WARN].value, BUFSIZ, "%d", nbar.cntdwn);
+  (void)strncpy (opt[CMD].value, nbar.cmd, BUFSIZ);
 
   l = strlen (opt[SHOW].name);
   x = x_pos + x_offset + l;
   mvwprintw (optwin, y_pos, x_pos, "[1] %s", opt[SHOW].name);
   erase_window_part (optwin, x, y_pos, maxcol, y_pos);
-  print_bool_option_incolor (optwin, nbar->show, y_pos, x);
+  print_bool_option_incolor (optwin, nbar.show, y_pos, x);
   mvwprintw (optwin, y_pos + 1, x_pos, opt[SHOW].desc);
 
   for (i = 1; i < NB_OPT; i++)
@@ -501,7 +504,7 @@ notify_print_options (WINDOW *optwin, int col)
 	mvwprintw (optwin, y, x, "%s", opt[i].value);
       else
 	{
-	  strncpy (buf, opt[i].value, maxlen - 1);
+	  (void)strncpy (buf, opt[i].value, maxlen - 1);
 	  buf[maxlen - 1] = '\0';
 	  mvwprintw (optwin, y, x, "%s...", buf);
 	}
@@ -509,7 +512,7 @@ notify_print_options (WINDOW *optwin, int col)
       mvwprintw (optwin, y + 1, x_pos, opt[i].desc);
     }
 
-  pthread_mutex_unlock (&nbar->mutex);
+  pthread_mutex_unlock (&nbar.mutex);
   wmove (win[STA].p, 1, 0);
   wnoutrefresh (optwin);
   doupdate ();
@@ -533,8 +536,11 @@ notify_config_bar (void)
   char *cmd_str = _("Enter the notification command ");
   int ch = 0, change_win = 1;
 
-  buf = (char *) malloc (BUFSIZ);
-  snprintf (label, BUFSIZ, _("CalCurse %s | notify-bar options"), VERSION);
+  buf = mem_malloc (BUFSIZ);
+  (void)snprintf (label, BUFSIZ, _("CalCurse %s | notify-bar options"),
+                  VERSION);
+  
+  conf_win.p = 0;
   custom_confwin_init (&conf_win, label);
 
   while (ch != 'q')
@@ -556,71 +562,70 @@ notify_config_bar (void)
 	  custom_confwin_init (&conf_win, label);
 	  break;
 	case '1':
-	  pthread_mutex_lock (&nbar->mutex);
-	  nbar->show = !nbar->show;
-	  pthread_mutex_unlock (&nbar->mutex);
+	  pthread_mutex_lock (&nbar.mutex);
+	  nbar.show = !nbar.show;
+	  pthread_mutex_unlock (&nbar.mutex);
 	  if (notify_bar ())
 	    notify_start_main_thread ();
 	  else
 	    notify_stop_main_thread ();
-	  delwin (conf_win.p);
 	  change_win = 1;
 	  break;
 	case '2':
 	  status_mesg (date_str, "");
-	  pthread_mutex_lock (&nbar->mutex);
-	  strncpy (buf, nbar->datefmt, strlen (nbar->datefmt) + 1);
-	  pthread_mutex_unlock (&nbar->mutex);
+	  pthread_mutex_lock (&nbar.mutex);
+	  (void)strncpy (buf, nbar.datefmt, strlen (nbar.datefmt) + 1);
+	  pthread_mutex_unlock (&nbar.mutex);
 	  if (updatestring (win[STA].p, &buf, 0, 1) == 0)
 	    {
-	      pthread_mutex_lock (&nbar->mutex);
-	      strncpy (nbar->datefmt, buf, strlen (buf) + 1);
-	      pthread_mutex_unlock (&nbar->mutex);
+	      pthread_mutex_lock (&nbar.mutex);
+	      (void)strncpy (nbar.datefmt, buf, strlen (buf) + 1);
+	      pthread_mutex_unlock (&nbar.mutex);
 	    }
 	  change_win = 0;
 	  break;
 	case '3':
 	  status_mesg (time_str, "");
-	  pthread_mutex_lock (&nbar->mutex);
-	  strncpy (buf, nbar->timefmt, strlen (nbar->timefmt) + 1);
-	  pthread_mutex_unlock (&nbar->mutex);
+	  pthread_mutex_lock (&nbar.mutex);
+	  (void)strncpy (buf, nbar.timefmt, strlen (nbar.timefmt) + 1);
+	  pthread_mutex_unlock (&nbar.mutex);
 	  if (updatestring (win[STA].p, &buf, 0, 1) == 0)
 	    {
-	      pthread_mutex_lock (&nbar->mutex);
-	      strncpy (nbar->timefmt, buf, strlen (buf) + 1);
-	      pthread_mutex_unlock (&nbar->mutex);
+	      pthread_mutex_lock (&nbar.mutex);
+	      (void)strncpy (nbar.timefmt, buf, strlen (buf) + 1);
+	      pthread_mutex_unlock (&nbar.mutex);
 	    }
 	  change_win = 0;
 	  break;
 	case '4':
 	  status_mesg (count_str, "");
-	  pthread_mutex_lock (&nbar->mutex);
-	  printf (buf, "%d", nbar->cntdwn);
-	  pthread_mutex_unlock (&nbar->mutex);
+	  pthread_mutex_lock (&nbar.mutex);
+	  printf (buf, "%d", nbar.cntdwn);
+	  pthread_mutex_unlock (&nbar.mutex);
 	  if (updatestring (win[STA].p, &buf, 0, 1) == 0 &&
 	      is_all_digit (buf) && atoi (buf) >= 0 && atoi (buf) <= DAYINSEC)
 	    {
-	      pthread_mutex_lock (&nbar->mutex);
-	      nbar->cntdwn = atoi (buf);
-	      pthread_mutex_unlock (&nbar->mutex);
+	      pthread_mutex_lock (&nbar.mutex);
+	      nbar.cntdwn = atoi (buf);
+	      pthread_mutex_unlock (&nbar.mutex);
 	    }
 	  change_win = 0;
 	  break;
 	case '5':
 	  status_mesg (cmd_str, "");
-	  pthread_mutex_lock (&nbar->mutex);
-	  strncpy (buf, nbar->cmd, strlen (nbar->cmd) + 1);
-	  pthread_mutex_unlock (&nbar->mutex);
+	  pthread_mutex_lock (&nbar.mutex);
+	  (void)strncpy (buf, nbar.cmd, strlen (nbar.cmd) + 1);
+	  pthread_mutex_unlock (&nbar.mutex);
 	  if (updatestring (win[STA].p, &buf, 0, 1) == 0)
 	    {
-	      pthread_mutex_lock (&nbar->mutex);
-	      strncpy (nbar->cmd, buf, strlen (buf) + 1);
-	      pthread_mutex_unlock (&nbar->mutex);
+	      pthread_mutex_lock (&nbar.mutex);
+	      (void)strncpy (nbar.cmd, buf, strlen (buf) + 1);
+	      pthread_mutex_unlock (&nbar.mutex);
 	    }
 	  change_win = 0;
 	  break;
 	}
     }
-  free (buf);
+  mem_free (buf);
   delwin (conf_win.p);
 }
