@@ -1,8 +1,8 @@
-/*	$calcurse: utils.c,v 1.61 2008/12/20 19:27:31 culot Exp $	*/
+/*	$calcurse: utils.c,v 1.68 2009/01/24 14:44:25 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2008 Frederic Culot
+ * Copyright (c) 2004-2009 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,32 +31,47 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <sys/types.h>
-#include <math.h>
 #include <errno.h>
 
+#include "utils.h"
 #include "i18n.h"
 #include "wins.h"
 #include "custom.h"
 #include "keys.h"
-#include "utils.h"
-
-#define NB_CAL_CMDS	24	/* number of commands while in cal view */
-#define NB_APP_CMDS	29	/* same thing while in appointment view */
-#define NB_TOD_CMDS	29	/* same thing while in todo view */
-#define TOTAL_CMDS	NB_CAL_CMDS + NB_APP_CMDS + NB_TOD_CMDS
-#define CMDS_PER_LINE	6	/* max number of commands per line */  
-
-static unsigned status_page;
+#include "io.h"
+#include "recur.h"
+#include "event.h"
+#include "apoint.h"
+#include "todo.h"
+#include "day.h"
+#include "keys.h"
+#include "mem.h"
 
 /* General routine to exit calcurse properly. */
 void
 exit_calcurse (int status)
 {
-  clear ();
-  refresh ();
-  endwin ();
-  ui_mode = UI_CMDLINE;
+  if (ui_mode == UI_CURSES)
+    {
+      clear ();
+      refresh ();
+      endwin ();
+      ui_mode = UI_CMDLINE;
+    }
   calendar_stop_date_thread ();
+  io_stop_psave_thread ();
+  day_free_list ();
+  event_llist_free ();
+  event_free_bkp (ERASE_FORCE);
+  apoint_llist_free ();
+  apoint_free_bkp (ERASE_FORCE);
+  recur_apoint_llist_free ();
+  recur_event_llist_free ();
+  recur_apoint_free_bkp (ERASE_FORCE);
+  recur_event_free_bkp (ERASE_FORCE);
+  todo_free_list ();
+  keys_free ();
+  mem_stats ();
   exit (status);
 }
 
@@ -66,13 +81,16 @@ fatalbox (const char *errmsg)
 {
   WINDOW *errwin;
   char *label = _("/!\\ INTERNAL ERROR /!\\");
-    char *reportmsg = _("Please report the following bug:");
+  char *reportmsg = _("Please report the following bug:");
   const int WINROW = 10;
   const int WINCOL = col - 2;
   const int MSGLEN = WINCOL - 2;
   char msg[MSGLEN];
 
-  strncpy (msg, errmsg, MSGLEN);
+  if (errmsg == 0)
+    return;
+  
+  (void)strncpy (msg, errmsg, MSGLEN);
   errwin = newwin (WINROW, WINCOL, (row - WINROW) / 2, (col - WINCOL) / 2);
   custom_apply_attr (errwin, ATTR_HIGHEST);
   box (errwin, 0, 0);
@@ -86,6 +104,32 @@ fatalbox (const char *errmsg)
   doupdate ();
 }
 
+void
+warnbox (const char *msg)
+{
+  WINDOW *warnwin;
+  char *label = "/!\\";
+  const int WINROW = 10;  
+  const int WINCOL = col - 2;
+  const int MSGLEN = WINCOL - 2;
+  char displmsg[MSGLEN];
+
+  if (msg == 0)
+    return;
+  
+  (void)strncpy (displmsg, msg, MSGLEN);
+  warnwin = newwin (WINROW, WINCOL, (row - WINROW) / 2, (col - WINCOL) / 2);
+  custom_apply_attr (warnwin, ATTR_HIGHEST);
+  box (warnwin, 0, 0);
+  wins_show (warnwin, label);
+  mvwprintw (warnwin, 5, (WINCOL - strlen (displmsg)) / 2, "%s", displmsg);
+  custom_remove_attr (warnwin, ATTR_HIGHEST);
+  wrefresh (warnwin);
+  (void)wgetch (warnwin);
+  delwin (warnwin);
+  doupdate ();
+}
+
 /* 
  * Print a message in the status bar.
  * Message texts for first line and second line are to be provided.
@@ -93,20 +137,13 @@ fatalbox (const char *errmsg)
 void
 status_mesg (char *mesg_line1, char *mesg_line2)
 {
-  erase_status_bar ();
+  wins_erase_status_bar ();
   custom_apply_attr (win[STA].p, ATTR_HIGHEST);
   mvwprintw (win[STA].p, 0, 0, mesg_line1);
   mvwprintw (win[STA].p, 1, 0, mesg_line2);
   custom_remove_attr (win[STA].p, ATTR_HIGHEST);
 }
 
-/* Erase status bar. */
-void
-erase_status_bar (void)
-{
-  erase_window_part (win[STA].p, 0, 0, col, STATUSHEIGHT);
-}
-
 /* Erase part of a window. */
 void
 erase_window_part (WINDOW *win, int first_col, int first_row, int last_col,
@@ -140,7 +177,7 @@ popup (int pop_row, int pop_col, int pop_y, int pop_x, char *title, char *msg,
     mvwprintw (popup_win, MSGXPOS, (pop_col - strlen (msg)) / 2, "%s", msg);
   custom_apply_attr (popup_win, ATTR_HIGHEST);
   box (popup_win, 0, 0);
-  snprintf (label, BUFSIZ, "%s", title);
+  (void)snprintf (label, BUFSIZ, "%s", title);
   wins_show (popup_win, label);
   if (hint)
     mvwprintw (popup_win, pop_row - 2, pop_col - (strlen (any_key) + 1), "%s",
@@ -363,18 +400,20 @@ updatestring (WINDOW *win, char **str, int x, int y)
   char *newstr;
   int escape, len = strlen (*str) + 1;
 
-  newstr = (char *) malloc (BUFSIZ);
+  EXIT_IF (len > BUFSIZ, _("Internal error: line too long"));
+  
+  newstr = mem_malloc (BUFSIZ);
   (void) memcpy (newstr, *str, len);
   escape = getstring (win, newstr, BUFSIZ, x, y);
   if (!escape)
     {
       len = strlen (newstr) + 1;
-      *str = (char *) realloc (*str, len);
+      *str = mem_realloc (*str, len);
       EXIT_IF (*str == 0, _("out of memory"));
       (void) memcpy (*str, newstr, len);
     }
-  free (newstr);
-  return (escape);
+  mem_free (newstr);
+  return escape;
 }
 
 /* checks if a string is only made of digits */
@@ -392,78 +431,39 @@ is_all_digit (char *string)
       digit++;
   if (digit == strlen (string))
     all_digit = 1;
-  return (all_digit);
+  return all_digit;
 }
 
-/* 
- * Draws the status bar. 
- * To add a keybinding, insert a new binding_t item, add it in the *binding
- * table, and update the NB_CAL_CMDS, NB_APP_CMDS or NB_TOD_CMDS defines,
- * depending on which panel the added keybind is assigned to.
- */
-void
-status_bar (void)
+/* Given an item date expressed in seconds, return its start time in seconds. */
+long
+get_item_time (long date)
 {
-#define NB_PANELS	3	/* 3 panels: CALENDAR, APPOINTMENT, TODO */
-  window_e which_pan;
-  int start, end;
-  const int pos[NB_PANELS + 1] =
-      { 0, NB_CAL_CMDS, NB_CAL_CMDS + NB_APP_CMDS, TOTAL_CMDS };
+  return (long)(get_item_hour (date) * HOURINSEC
+                + get_item_min (date) * MININSEC);
+}
 
-  binding_t help   = {_("Help"),     KEY_GENERIC_HELP};
-  binding_t quit   = {_("Quit"),     KEY_GENERIC_QUIT};
-  binding_t save   = {_("Save"),     KEY_GENERIC_SAVE};
-  binding_t chgvu  = {_("Chg View"), KEY_GENERIC_CHANGE_VIEW};
-  binding_t import = {_("Import"),   KEY_GENERIC_IMPORT};  
-  binding_t export = {_("Export"),   KEY_GENERIC_EXPORT};
-  binding_t togo   = {_("Go to"),    KEY_GENERIC_GOTO};
-  binding_t othr   = {_("OtherCmd"), KEY_GENERIC_OTHER_CMD};
-  binding_t conf   = {_("Config"),   KEY_GENERIC_CONFIG_MENU};  
-  binding_t draw   = {_("Redraw"),   KEY_GENERIC_REDRAW};
-  binding_t appt   = {_("Add Appt"), KEY_GENERIC_ADD_APPT};
-  binding_t todo   = {_("Add Todo"), KEY_GENERIC_ADD_TODO};
-  binding_t gnday  = {_("+1 Day"),   KEY_GENERIC_NEXT_DAY};
-  binding_t gpday  = {_("-1 Day"),   KEY_GENERIC_PREV_DAY};
-  binding_t gnweek = {_("+1 Week"),  KEY_GENERIC_NEXT_WEEK};
-  binding_t gpweek = {_("-1 Week"),  KEY_GENERIC_PREV_WEEK};
-  binding_t today  = {_("Today"),    KEY_GENERIC_GOTO_TODAY};
-  binding_t up     = {_("Up"),       KEY_MOVE_UP};
-  binding_t down   = {_("Down"),     KEY_MOVE_DOWN};  
-  binding_t left   = {_("Left"),     KEY_MOVE_LEFT};
-  binding_t right  = {_("Right"),    KEY_MOVE_RIGHT};  
-  binding_t weekb  = {_("beg Week"), KEY_START_OF_WEEK};
-  binding_t weeke  = {_("end Week"), KEY_END_OF_WEEK};
-  binding_t add    = {_("Add Item"), KEY_ADD_ITEM};
-  binding_t del    = {_("Del Item"), KEY_DEL_ITEM};
-  binding_t edit   = {_("Edit Itm"), KEY_EDIT_ITEM};
-  binding_t view   = {_("View"),     KEY_VIEW_ITEM};  
-  binding_t flag   = {_("Flag Itm"), KEY_FLAG_ITEM};
-  binding_t rept   = {_("Repeat"),   KEY_REPEAT_ITEM};
-  binding_t enote  = {_("EditNote"), KEY_EDIT_NOTE};
-  binding_t vnote  = {_("ViewNote"), KEY_VIEW_NOTE};
-  binding_t rprio  = {_("Prio.+"),   KEY_RAISE_PRIORITY};
-  binding_t lprio  = {_("Prio.-"),   KEY_LOWER_PRIORITY};  
-  
-  binding_t *binding[TOTAL_CMDS] = {
-    /* calendar keys */
-    &help, &quit, &save, &chgvu, &import, &export, &up, &down, &left, &right,
-    &togo, &othr, &weekb, &weeke, &conf, &draw, &appt, &todo, &gnday, &gpday,
-    &gnweek, &gpweek, &today, &othr,
-    /* appointment keys */
-    &help, &quit, &save, &chgvu, &import, &export, &add, &del, &edit, &view,
-    &draw, &othr, &rept, &flag, &enote, &vnote, &up, &down, &gnday, &gpday,
-    &gnweek, &gpweek, &togo, &othr, &today, &conf, &appt, &todo, &othr,
-    /* todo keys */
-    &help, &quit, &save, &chgvu, &import, &export, &add, &del, &edit, &view,
-    &draw, &othr, &rprio, &lprio, &enote, &vnote, &up, &down, &gnday, &gpday,
-    &gnweek, &gpweek, &togo, &othr, &today, &conf, &appt, &todo, &othr
-  };
+int
+get_item_hour (long date)
+{
+  struct tm *lt;
+  time_t t;
 
-  /* Drawing the keybinding with attribute and label without. */
-  which_pan = wins_slctd ();
-  start = pos[which_pan] + 2 * KEYS_CMDS_PER_LINE * (status_page - 1);
-  end = MIN (start + 2 * KEYS_CMDS_PER_LINE, pos[which_pan + 1]);
-  keys_display_bindings_bar (win[STA].p, binding, start, end);
+  t = (time_t)date;
+  lt = localtime (&t);
+
+  return lt->tm_hour;
+}
+
+int
+get_item_min (long date)
+{
+  struct tm *lt;
+  time_t t;
+
+  t = (time_t)date;
+  lt = localtime (&t);
+
+  return lt->tm_min;
 }
 
 long
@@ -488,7 +488,7 @@ date2sec (date_t day, unsigned hour, unsigned min)
   tstart = mktime (&start);
   EXIT_IF (tstart == -1, _("failure in mktime"));
 
-  return (tstart);
+  return tstart;
 }
 
 /* Return a string containing the hour of a given date in seconds. */
@@ -502,9 +502,9 @@ date_sec2hour_str (long sec)
 
   t = sec;
   lt = localtime (&t);
-  timestr = (char *) malloc (TIME_LEN);
-  snprintf (timestr, TIME_LEN, "%02u:%02u", lt->tm_hour, lt->tm_min);
-  return (timestr);
+  timestr = (char *) mem_malloc (TIME_LEN);
+  (void)snprintf (timestr, TIME_LEN, "%02u:%02u", lt->tm_hour, lt->tm_min);
+  return timestr;
 }
 
 /* Return a string containing the date, given a date in seconds. */
@@ -515,10 +515,10 @@ date_sec2date_str (long sec, char *datefmt)
   time_t t;
   char *datestr;
 
-  datestr = (char *) malloc (sizeof (char) * BUFSIZ);
+  datestr = (char *) mem_calloc (BUFSIZ, sizeof (char));
 
   if (sec == 0)
-    snprintf (datestr, BUFSIZ, "0");
+    (void)snprintf (datestr, BUFSIZ, "0");
   else
     {
       t = sec;
@@ -634,7 +634,7 @@ check_time (char *string)
   if (((strlen (string) == 2) || (strlen (string) == 3))
       && (isdigit (string[0]) != 0) && (isdigit (string[1]) != 0))
     {
-      strncpy (minutes, string, 2);
+      (void)strncpy (minutes, string, 2);
       if (atoi (minutes) >= 0)
 	ok = 2;			/* [MM] format */
     }
@@ -642,7 +642,7 @@ check_time (char *string)
 	   && (isdigit (string[2]) != 0) && (isdigit (string[3]) != 0)
            && (string[1] == ':'))
     {
-      strncpy (hour, string, 1);
+      (void)strncpy (hour, string, 1);
       strncpy (minutes, string + 2, 2);
       if ((atoi (hour) <= 24) && (atoi (hour) >= 0)
 	  && (atoi (minutes) < MININSEC) && (atoi (minutes) >= 0))
@@ -709,41 +709,6 @@ item_in_popup (char *saved_a_start, char *saved_a_end, char *msg,
   delwin (popup_win);
 }
 
-/* Reset the status bar page. */
-void
-reset_status_page (void)
-{
-  status_page = 1;
-}
-
-/* Update the status bar page number to display other commands. */
-void
-other_status_page (int panel)
-{
-  int nb_item = 0, max_page;
-
-  switch (panel)
-    {
-    case CAL:
-      nb_item = NB_CAL_CMDS;
-      break;
-    case APP:
-      nb_item = NB_APP_CMDS;
-      break;
-    case TOD:
-      nb_item = NB_TOD_CMDS;
-      break;
-    default:
-      EXIT (_("unknown panel"));
-      /* NOTREACHED */
-    }
-  max_page = ceil (nb_item / (2 * CMDS_PER_LINE + 1)) + 1;
-  if (status_page < max_page)
-    status_page++;
-  else
-    status_page = 1;
-}
-
 /* Returns the beginning of current day in seconds from 1900. */
 long
 get_today (void)
@@ -834,7 +799,7 @@ new_tempfile (const char *prefix, int trailing_len)
   if (prefix_len + trailing_len >= BUFSIZ)
     return (NULL);
   memcpy (fullname, prefix, prefix_len);
-  memset (fullname + prefix_len, 'X', trailing_len);
+  (void)memset (fullname + prefix_len, 'X', trailing_len);
   fullname[prefix_len + trailing_len] = '\0';
   if ((fd = mkstemp (fullname)) == -1 || (file = fdopen (fd, "w+")) == NULL)
     {
@@ -848,7 +813,7 @@ new_tempfile (const char *prefix, int trailing_len)
     }
   fclose (file);
 
-  return (strdup (fullname + prefix_len));
+  return mem_strdup (fullname + prefix_len);
 }
 
 /* Erase a note previously attached to a todo, event or appointment. */
@@ -861,11 +826,11 @@ erase_note (char **note, erase_flag_e flag)
     return;
   if (flag != ERASE_FORCE_KEEP_NOTE)
     {
-      snprintf (fullname, BUFSIZ, "%s%s", path_notes, *note);
+      (void)snprintf (fullname, BUFSIZ, "%s%s", path_notes, *note);
       if (unlink (fullname) != 0)
         EXIT (_("could not remove note"));
     }
-  free (*note);
+  mem_free (*note);
   *note = NULL;
 }
 
@@ -879,40 +844,57 @@ parse_date (char *date_string, int datefmt, int *year, int *month, int *day)
 {
   int in1, in2, in3;
   int lyear, lmonth, lday;
-  if (date_string == NULL)
-    return (0);
-  if (sscanf (date_string, "%d / %d / %d", &in1, &in2, &in3) < 3)
-    return (0);
+
+  if (date_string == 0)
+    return 0;
+
+  if (datefmt == DATEFMT_ISO)
+    {
+      if (sscanf (date_string, "%d - %d - %d", &in1, &in2, &in3) < 3)
+        return 0;
+    }
+  else
+    {
+      if (sscanf (date_string, "%d / %d / %d", &in1, &in2, &in3) < 3)
+        return 0;
+    }
   switch (datefmt)
     {
-    case 1:
+    case DATEFMT_MMDDYYYY:
       lmonth = in1;
       lday = in2;
       lyear = in3;
       break;
-    case 2:
+    case DATEFMT_DDMMYYYY:
       lday = in1;
       lmonth = in2;
       lyear = in3;
       break;
-    case 3:
+    case DATEFMT_YYYYMMDD:
+    case DATEFMT_ISO:      
       lyear = in1;
       lmonth = in2;
       lday = in3;
       break;
     default:
-      return (0);
+      return 0;
     }
-  if (lyear < 1 || lyear > 9999 || lmonth < 1 || lmonth > 12 || lday < 1
-      || lday > 31)
-    return (0);
+  
+  if (lyear < 1 || lyear > 9999
+      || lmonth < 1 || lmonth > 12
+      || lday < 1 || lday > 31)
+    return 0;
+  
   if (year != NULL)
     *year = lyear;
+  
   if (month != NULL)
     *month = lmonth;
+  
   if (day != NULL)
     *day = lday;
-  return (1);
+  
+  return 1;
 }
 
 char *
@@ -926,10 +908,10 @@ str_toupper (char *s)
 }
 
 void
-mem_free (void *ptr)
+file_close (FILE *f, const char *pos)
 {
-  if (!ptr)
-    return;
-  free (ptr);
-  ptr = NULL;
+  int ret;
+  
+  ret = fclose (f);
+  EXIT_IF (ret != 0, _("Error when closing file at %s"), pos);
 }
