@@ -1,4 +1,4 @@
-/*	$calcurse: vars.c,v 1.11 2008/11/16 17:42:53 culot Exp $	*/
+/*	$calcurse: vars.c,v 1.14 2009/01/02 22:28:54 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -32,6 +32,7 @@
 #include "custom.h"
 #include "wins.h"
 #include "keys.h"
+#include "mem.h"
 #include "vars.h"
 
 /*
@@ -91,10 +92,10 @@ char path_notes[] = "";
 char path_keys[] = "";
 
 /* Variable to handle pads. */
-struct pad_s *apad;
+struct pad_s apad;
 
 /* Variable to store notify-bar settings. */
-struct nbar_s *nbar;
+struct nbar_s nbar;
 
 /*
  * Variables init 
@@ -110,9 +111,10 @@ vars_init (conf_t *conf)
   conf->confirm_quit = true;
   conf->confirm_delete = true;
   conf->auto_save = true;
+  conf->periodic_save = 0;
   conf->skip_system_dialogs = false;
   conf->skip_progress_bar = false;
-  strncpy (conf->output_datefmt, "%D", 3);
+  (void)strncpy (conf->output_datefmt, "%D", 3);
   conf->input_datefmt = 1;
 
   /* Default external editor and pager */
@@ -133,9 +135,8 @@ vars_init (conf_t *conf)
   calendar_set_first_day_of_week (MONDAY);
 
   /* Pad structure to scroll text inside the appointment panel */
-  apad = (struct pad_s *) malloc (sizeof (struct pad_s));
-  apad->length = 1;
-  apad->first_onscreen = 0;
+  apad.length = 1;
+  apad.first_onscreen = 0;
 
   /* Attribute definitions for color and non-color terminals */
   custom_init_attr ();
