@@ -1,8 +1,8 @@
-/*	$calcurse: vars.h,v 1.26 2008/11/16 17:42:53 culot Exp $	*/
+/*	$calcurse: vars.h,v 1.31 2009/01/23 21:09:21 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
- * Copyright (c) 2004-2008 Frederic Culot
+ * Copyright (c) 2004-2009 Frederic Culot
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,20 @@
 #ifndef CALCURSE_VARS_H
 #define CALCURSE_VARS_H
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
+#ifdef HAVE_NCURSES_H
 #include <ncurses.h>
+#elif defined HAVE_NCURSES_NCURSES_H
+#include <ncurses/ncurses.h>
+#elif defined HAVE_NCURSESW_NCURSES_H
+#include <ncursesw/ncurses.h>
+#else
+#error "Missing ncurses header. Aborting..."
+#endif
+
 #include <pthread.h>
 #include <stdbool.h>
 
@@ -59,11 +72,24 @@
 #define STATUSHEIGHT	2
 #define	NOTESIZ		6
 
-#define DATEFMT(datefmt) (datefmt == 1 ? "%m/%d/%Y" : \
-		(datefmt == 2 ? "%d/%m/%Y" : "%Y/%m/%d"))
+enum {
+  DATEFMT_MMDDYYYY = 1,
+  DATEFMT_DDMMYYYY,
+  DATEFMT_YYYYMMDD,
+  DATEFMT_ISO,
+  DATE_FORMATS
+};
 
-#define DATEFMT_DESC(datefmt) (datefmt == 1 ? _("mm/dd/yyyy") : \
-		(datefmt == 2 ? _("dd/mm/yyyy") : _("yyyy/mm/dd")))
+#define DATEFMT(datefmt) (datefmt == DATEFMT_MMDDYYYY ? "%m/%d/%Y" :          \
+		(datefmt == DATEFMT_DDMMYYYY ? "%d/%m/%Y" :                   \
+                (datefmt == DATEFMT_YYYYMMDD ? "%Y/%m/%d" : "%Y-%m-%d")))
+
+#define DATEFMT_DESC(datefmt) (datefmt == DATEFMT_MMDDYYYY ?                  \
+                               _("mm/dd/yyyy") :                              \
+		               (datefmt == DATEFMT_DDMMYYYY ?                 \
+                               _("dd/mm/yyyy") :                              \
+                               (datefmt == DATEFMT_YYYYMMDD ?                 \
+                               _("yyyy/mm/dd") : _("yyyy-mm-dd"))))
 
 typedef enum {
   UI_CURSES,
@@ -94,15 +120,16 @@ struct nbar_s
 /* General configuration variables */
 typedef struct
 {
-  bool  auto_save;
-  bool  confirm_quit;
-  bool  confirm_delete;
-  bool  skip_system_dialogs;
-  bool  skip_progress_bar;
-  char *editor;
-  char *pager;
-  char  output_datefmt[BUFSIZ];	/* format for displaying date */
-  int   input_datefmt;		/* format for reading date */
+  bool       auto_save;
+  unsigned   periodic_save;
+  bool       confirm_quit;
+  bool       confirm_delete;
+  bool       skip_system_dialogs;
+  bool       skip_progress_bar;
+  char      *editor;
+  char      *pager;
+  char       output_datefmt[BUFSIZ];	/* format for displaying date */
+  int        input_datefmt;		/* format for reading date */
 }
 conf_t;
 
@@ -118,9 +145,9 @@ extern char           path_apts[BUFSIZ];
 extern char           path_conf[BUFSIZ];
 extern char           path_keys[BUFSIZ];
 extern char           path_notes[BUFSIZ];
-extern struct pad_s  *apad;
-extern struct nbar_s *nbar;
+extern struct pad_s   apad;
+extern struct nbar_s  nbar;
 
-void vars_init (conf_t *conf);
+void vars_init (conf_t *);
 
 #endif /* CALCURSE_VARS_H */
